@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2000-2004
+  Copyright (C) 2000-2006
 
   Code contributed by Greg Collecutt, Joseph Hope and Paul Cochrane
 
@@ -21,37 +21,36 @@
 */
 
 /*
-  $Id: main.cc,v 1.43 2005/10/26 03:46:28 joehope Exp $
+  $Id: main.cc 1361 2007-06-21 01:31:44Z joehope $
 */
 
 /*!
-  @mainpage Documentation for xmds-1.5-2
-
-  @author Paul Cochrane
+  @mainpage Documentation for xmds-1.5-4
 
   @section intro Introduction
 
   xmds is the eXtensible Multi-Dimensional Simulator.
 
-  XMDS is a code generator that integrates equations. You write them
-  down in human readable form in a XML file, and it goes away and
-  writes and compiles a C++ program that integrates those equations as
-  fast as it can possibly be done in your architecture.
+  XMDS is a code generator that integrates equations. You write them down in
+  human readable form in a XML file, and it goes away and writes and
+  compiles a C++ program that integrates those equations as fast as it can
+  possibly be done in your architecture.
 
-  Originally written by Greg Collecutt (and the majority of the code base is still
-  due to him), however is now maintained by Paul Cochrane.
+  Originally written by Greg Collecutt (and the majority of the code base is
+  still due to him), however is now maintained by Joe Hope and the xmds
+  development team.
 
   @section install Installation
 
-  Download the source tarball from http://www.xmds.org, unpack, and run the configure script in
-  the xmds directory.<br>
+  Download the source tarball from http://www.xmds.org, unpack, and run the 
+  configure script in the xmds directory.<br>
   (as root, to be installed into /usr/local/bin)<br>
   ./configure<br>
   (as a user, to be installed the bin/ directory in your home directory)<br>
   ./configure --with-user<br>
 
-  For more details you can also read the INSTALL file, and even the hand-written
-  documentation.
+  For more details you can also read the INSTALL file, and even the 
+  hand-written documentation.
 */
 
 /*! 
@@ -63,20 +62,22 @@
 
 // This is the main entry routine for xmds
 
-#include<config.h>
-#include<xmlbasics.h>
-#include<dom3.h>
-#include<kissdom.h>
-#include<xmlparser.h>
-#include<xmdsutils.h>
-#include<xmdsclasses.h>
-#include<string>
-#include<cstdio>
-#include<cstdlib>
-#include<iostream>
-#include<fstream>
-#include<vector>
-#include<getopt_xmds.h>
+#include <config.h>
+#include <xmlbasics.h>
+#include <dom3.h>
+#include <kissdom.h>
+#include <xmlparser.h>
+#include <xmdsutils.h>
+#include <xmdsclasses.h>
+#include <string>
+#include <cstring>
+#include <cstdio>
+#include <cstdlib>
+#include <iostream>
+#include <fstream>
+#include <vector>
+#include <getopt_xmds.h>
+#include "version.h"
 
 using namespace std;
 
@@ -84,33 +85,27 @@ using namespace std;
 
 //! Displays xmds usage
 void display_usage() {
-  printf("\n");
-  printf("This is xmds version ");
-  printf(VERSION);
-  printf(" release ");
-  printf(RELEASE);
-  printf("\n");
-  printf("\n");
-  printf("      using C compiler ");
-  printf(XMDS_CC);
-  printf("\n");
-  if(strcmp(MPICC,"")) {
-    printf("    (and C compiler %s\n",MPICC);
-    printf(" for parallel work)\n");
+  cout << "\n" <<
+    "This is xmds version " << VERSION << " release " << RELEASE << " (" << REVISION << ")\n\n" << 
+    "      using C compiler " << XMDS_CC;
+  if (strcmp(MPICC, "")) {
+    cout << " (and C compiler " << MPICC << " for parallel work)\n";
   }
-  printf(
+  cout <<
          "\n"
          "Usage: xmds [options] infile\n"
-	 "Options:\n"
+         "Options:\n"
          "  infile:         required,  The input file\n"
-	 "  -h/--help:      optional,  Display this information\n"
+         "  -h/--help:      optional,  Display this information\n"
          "  -v/--verbose:   optional,  Verbose mode\n"
-         "  -n/--nocompile: optional,  Turns off automatic compilation of simulation\n"
-         "  -t/--template:  optional,  Outputs an xmds template either to the terminal,\n"
+         "  -n/--nocompile: optional,  "
+                           "Turns off automatic compilation of simulation\n"
+         "  -t/--template:  optional,  "
+                           "Outputs an xmds template either to the terminal,\n"
          "                               or to an optionally specified file\n"
          "\n"
-	 "For further help, please see http://www.xmds.org\n"
-         );
+         "For further help, please see http://www.xmds.org\n"
+         ;
 }
 
 // ********************************************************************
@@ -128,80 +123,98 @@ void display_usage() {
   @param mpicflags      The C/C++ compiler flags for the MPI C/C++ compiler
   @param fftwlibs       The libraries for using fftw
   @param fftw_mpi_libs  The libraries necessary for using fftw with MPI
+  @param fftw3libs      The libraries for using fftw3
+  @param fftw3threadlibs The libraries for using fftw3 with threads
   @param verbose        Whether or not to print verbose information
   @param debug          Whether or not to print debugging information
 */
 int parsePrefs(ifstream &fPrefs, 
-               string &cc, string &cflags, string &clibs, string &cincludes, string &cthreadlibs, 
+               string &cc, string &cflags, string &clibs, string &cincludes, 
+               string &cthreadlibs, 
                string &mpicc, string &mpicflags, 
                string &fftwlibs, string &fftw_mpi_libs, 
+               string &fftw3libs, string &fftw3threadlibs,
                bool verbose, bool debug) {
   
-  /* 
-     I've thought of a better way to do this, but first I'll just get this version
-     going so that the feature is in, and then I'll go back and make it a bit more
-     elegant.
-     The idea is to read in each line of the prefs file individually (into a string),
-     and then process the line.  I'll need to read characters until I find an equals 
-     sign (ignoring spaces as I go), biff that into macroVar and then grab everything
-     else and put that into macroVarValue.  I could treat the string read in (ie the
-     line I'm parsing) as a stack and pop characters off it until the equals sign
-     is found, putting chars into macroVar and then put the rest (without equals sign
-     into macroVarValue.
-     Anyway, to be done...
+  /*! 
+     \todo
+     I've thought of a better way to do this, but first I'll just get this
+     version going so that the feature is in, and then I'll go back and make
+     it a bit more elegant.  The idea is to read in each line of the prefs
+     file individually (into a string), and then process the line.  I'll
+     need to read characters until I find an equals sign (ignoring spaces as
+     I go), biff that into macroVar and then grab everything else and put
+     that into macroVarValue.  I could treat the string read in (ie the line
+     I'm parsing) as a stack and pop characters off it until the equals sign
+     is found, putting chars into macroVar and then put the rest (without
+     equals sign into macroVarValue.  Anyway, to be done...
 
-     Also, I should make this into a function so that the code is only written the once!
+     Also, I should make this into a function so that the code is only
+     written the once!
   */
 
+  /*! 
+   * \todo this routine should really be part of xmdsUtility, and therefore
+   * an instance of an xmdsUtility object (or something like that), because
+   * there are far too many parameters being passed into the function, and
+   * they could be packaged more nicely into an object, rather than as
+   * params.  Anyway, something else to do....
+   */
+
+  //! \todo replace if (debug) etc... with debug() routine.
+
   // grab the text
-  char c;
+  char currentChar;
   string macroVar, macroVarValue, prefsString;
   bool commentCharFlag;
   prefsString = "";
-  while( !fPrefs.eof() ) {
+  while ( !fPrefs.eof() ) {
     // now we try the next line
     macroVar = "";
     macroVarValue = "";
     commentCharFlag = 0;
 
     // wait until we find the equals sign
-    while( (c = fPrefs.get()) != '=' && !fPrefs.eof() && !commentCharFlag) {
-      if (c == ' ') {
+    while ( (currentChar = fPrefs.get()) != '=' && 
+           !fPrefs.eof() && 
+           !commentCharFlag) {
+      if (currentChar == ' ') {
         if (debug) { cout << "space character before '=' found\n"; }
       }
-      else if (c == '#') {
+      else if (currentChar == '#') {
         if (debug) { cout << "comment character found (before '=' found)\n"; }
         commentCharFlag = 1;
-        while ( (c = fPrefs.get()) != '\n' && !fPrefs.eof() ) {
-          if (debug) {
-	      cout << "looping until end of line\n"; 
-	  } 
-          // cout << "c = " << c << "\n";
+        while ( (currentChar = fPrefs.get()) != '\n' && 
+                !fPrefs.eof() ) {
+          if (debug) { cout << "looping until end of line\n"; } 
         }
         break;
       }
       else {
-        macroVar = macroVar + c;
+        macroVar = macroVar + currentChar;
       }
     }
 
     // now loop until we find the return character
-    while( !commentCharFlag && (c = fPrefs.get()) != '\n' && !fPrefs.eof()) {
-      if (c == '#') {
+    while ( !commentCharFlag && 
+           (currentChar = fPrefs.get()) != '\n' && 
+           !fPrefs.eof()) {
+      if (currentChar == '#') {
         if (debug) { cout << "comment character found\n"; }
         commentCharFlag = 1;
-        while ( (c = fPrefs.get()) != '\n' && !fPrefs.eof() ) {
+        while ( (currentChar = fPrefs.get()) != '\n' && 
+                !fPrefs.eof() ) {
           if (debug) { cout << "looping until end of line\n"; }
-          // cout << "c = " << c << "\n";
         }
         break;
       }
-      macroVarValue = macroVarValue + c;
+      macroVarValue = macroVarValue + currentChar;
     }
-        
-    // cout << "macroVar = " + macroVar + "\n";
-    // cout << "macroVarValue = " + macroVarValue + "\n";
 
+    /*! 
+     * \todo this if-elseif sequence can be replaced by a switch-case
+     * sequence if the strings are replaced by enums or something similar
+     */
     // now do some assignments
     if (macroVar == "XMDS_CC") {
       cc = macroVarValue;
@@ -235,6 +248,14 @@ int parsePrefs(ifstream &fPrefs,
       fftwlibs = macroVarValue;
       if (verbose) { cout << "fftwlibs set to " + macroVarValue + "\n"; }
     }
+    else if (macroVar == "FFTW3_LIBS") {
+      fftw3libs = macroVarValue;
+      if (verbose) { cout << "fftw3libs set to " + macroVarValue + "\n"; }
+    }
+    else if (macroVar == "FFTW3_THREADLIBS") {
+      fftw3threadlibs = macroVarValue;
+      if (verbose) { cout << "fftw3threadlibs set to " + macroVarValue + "\n"; }
+    }
     else if (macroVar == "FFTW_MPI_LIBS") {
       fftw_mpi_libs = macroVarValue;
       if (verbose) { cout << "fftw_mpi_libs set to " + macroVarValue + "\n"; }
@@ -254,6 +275,8 @@ void outputTemplate(const char* outfilename) {
   // this is the template text, at present taken directly from the 
   // tutorial, tutTemplateStart.tex of the latex documentation
   // and relevant characters escaped so that they print out properly
+  //! \todo does the template text need to be updated???
+  //! \todo the templateText variable should be a string vector
   const char* templateText = 
     "<?xml version=\"1.0\"?>\n"
     "<simulation>\n"
@@ -348,30 +371,36 @@ void outputTemplate(const char* outfilename) {
     "  \n"
     "</simulation>\n";
     
-    // check to see if an output file was given
-  if(outfilename==0) {
+  // check to see if an output file was given
+  if (outfilename == 0) {
     // ok, so no input file specified, we therefore spit it out to stdout
-    printf("%s",templateText);
+    cout << templateText;
   }
-  else if (outfilename!=0) {
+  else if (outfilename != 0) {
     // ok, we have an input file, open it, biff out the string, close it
     // btw, why am I using the old C syntax for this and not C++???
+    //! \todo update file output syntax to C++ completely
     FILE *fp;
     fp = fopen(outfilename, "w");
     if (fp == NULL) {   
-	// make sure can actually open the file
-      printf("Unable to open output file: %s\n",outfilename);
-      printf("Sending output to stdout\n\n");
+      // make sure can actually open the file
+      cout << "Unable to open output file: " << outfilename << "\n";
+      cout << "Sending output to stdout\n\n";
       
-      printf("%s",templateText);
+      cout << templateText;
     }
     else { 
-      printf("This is xmds, version %s-%s\n", VERSION, RELEASE);
-      printf("Copyright 2000-2004 Greg Collecutt, Joseph Hope, Paul Cochrane and others\n");
-      printf("xmds is available from http://www.xmds.org\n\n");
-      printf("Writing a template to file with filename: %s\n",outfilename);
+      /*! 
+       * \todo the xmds version/copyright info is repeated too often
+       * put this into a routine!
+       */
+      cout << "This is xmds, version " << VERSION << "-" << RELEASE << " (" << REVISION <<")\n" <<
+        "Copyright 2000-2006 Greg Collecutt, Joseph Hope "<< 
+        "and the xmds-devel team\n" << 
+        "xmds is available from http://www.xmds.org\n\n" << 
+        "Writing a template to file with filename: " << outfilename << "\n";
       fprintf(fp, "%s", templateText);
-      printf("Done!\n");
+      cout << "Done!\n";
     }
     fclose(fp);
   }
@@ -379,12 +408,12 @@ void outputTemplate(const char* outfilename) {
 
 /* ******************************************************************** */
 
-bool debugFlag = 0;                  //!< Print debugging information about xmds processes
-bool xmlDebugFlag = 0;               //!< Print debugging information about xml parsing processes
-vector<string> simulationText; //!< The text of the xmds simulation script
-vector<string> simHeaderText;                //!< The text of the xmds script's header
-vector<string> simBodyText;                  //!< The text of the xmds script's body
-vector<string> simFooterText;                //!< The text of the xmds script's footer
+bool debugFlag = 0;             //!< Print debugging info about xmds processes
+bool xmlDebugFlag = 0;          //!< Print debugging info about xml parsing processes
+vector<string> simulationText;  //!< The text of the xmds simulation script
+vector<string> simHeaderText;   //!< The text of the xmds script's header
+vector<string> simBodyText;     //!< The text of the xmds script's body
+vector<string> simFooterText;   //!< The text of the xmds script's footer
 
 /*! 
   @brief The main routine.
@@ -394,8 +423,8 @@ vector<string> simFooterText;                //!< The text of the xmds script's
 */
 int main(
          int argc,
-         char **argv) {
-
+         char **argv)
+{
   bool verbose = 0;
   bool compileFlag = 1;
   bool templateGenFlag = 0;
@@ -404,93 +433,94 @@ int main(
   int resp;
   while (1) {
     static struct option long_options[] = 
-    {
-	{"help", no_argument, 0, 'h'},
-	{"verbose", no_argument, 0, 'v'},
-	{"debug", no_argument, 0, 'd'},
-	{"nocompile", no_argument, 0, 'n'},
-	{"template", optional_argument, 0, 't'},
-	{"xmldebug", no_argument, 0, 'x'},
-	{0,0,0,0}
+    { 
+      {"help", no_argument, 0, 'h'}, 
+      {"verbose", no_argument, 0, 'v'}, 
+      {"debug", no_argument, 0, 'd'}, 
+      {"nocompile", no_argument, 0, 'n'}, 
+      {"template", optional_argument, 0, 't'}, 
+      {"xmldebug", no_argument, 0, 'x'}, 
+      {0,0,0,0} 
     };
     int option_index = 0;
     resp = getopt_xmds_long(argc, argv, "hvdnxt", long_options, &option_index);
     if (resp == -1) {
-	break;
+      break;
+    }
+    switch (resp) { 
+      case 'h':
+        display_usage();
+        return 0;
+      case 'v':
+        verbose = 1;
+        break;
+      case 'd':
+        debugFlag = 1;
+        break;
+      case 'n':
+        compileFlag = 0;
+        break;
+      case 't':
+        templateGenFlag = 1;
+        break;
+      case 'x':
+        xmlDebugFlag = 1;
+        break;
+      default:
+        display_usage();
+        return 0;
     }
-    switch (resp) {
-	case 'h':
-	    display_usage();
-	    return 0;
-	case 'v':
-	    verbose = 1;
-	    break;
-	case 'd':
-	    debugFlag = 1;
-	    break;
-	case 'n':
-	    compileFlag = 0;
-	    break;
-	case 't':
-	    templateGenFlag = 1;
-	    break;
-	case 'x':
-	    xmlDebugFlag = 1;
-	    break;
-	default:
-	  display_usage();
-	  return 0;
-        }
   }
+
   // process non-option command line elements
   if (optind_xmds < argc) {
-      int fnameCount = 0;
-      while (optind_xmds < argc) {
-	  fnameCount++;
-	  if (fnameCount > 1) {
-	      // error, input file name already exists
-	      printf("Error: multiple input files not allowed\n\n");
-	      display_usage();
-	      return 0;
-	  }
-          // assign infilename pointer to the appropriate
-          // member of the argv array
-          infilename = argv[optind_xmds++];
+    int fnameCount = 0;
+    while (optind_xmds < argc) { 
+      fnameCount++; 
+      if (fnameCount > 1) {
+        // error, input file name already exists
+        printf("Error: multiple input files not allowed\n\n");
+        display_usage();
+        return 0;
       }
+
+      // assign infilename pointer to the appropriate
+      // member of the argv array
+      infilename = argv[optind_xmds++];
+    }
   }
 
   // if asked to make a template, then just spit it out, either to file
   // or to stdout and then return nicely
   if (templateGenFlag) {
-    // at present, we'll reuse the input file.  I intend to change this to
-    // use the getopt_xmds stuff later, so will have to do that at some stage.
+    /*! \todo 
+     * at present, we'll reuse the input file.  I intend to change this to
+     * use the getopt_xmds stuff later, so will have to do that at some stage.
+     */
     outputTemplate(infilename);
     return 0;
   }
 
-
   // check to see that an input file was given
-  if(infilename==0) {
+  if (infilename==0) {
     // error, no input file was specified
-    printf("Error: no input file specified!\n\n");
+    cout << "Error: no input file specified!\n\n";
     display_usage();
-    return 0;
+    return 1;
   }
   
-  if(verbose) {
-    printf("xmds: inputfile = '%s'\n",infilename);
+  if (verbose) {
+    cout << "xmds: inputfile = '" << infilename << "'\n";
   }
   
   // create the XMLParser
-
   XMLParser myXMLParser;
 
   // now load xmds script into the DOMImplementation
-
   Document* theDocument=0;
 
-  if(verbose) {
-    printf("Parsing file '%s' ...\n",infilename);
+  if (verbose) {
+    cout << "Parsing file '" << infilename << "' ...\n";
   }
 
   try {
@@ -500,29 +530,29 @@ int main(
     // it looks like it'll have to be in C for the most part
     FILE *fin;
     if ((fin = fopen(infilename,"r")) == NULL) {
-      printf("Can't open the input xmds script file: %s\n"
-	     "Exiting\n"
-	     ,infilename);
+      cout << "Can't open the input xmds script file: " << infilename
+        << "Exiting\n"; 
       return 1;  // and barf
     }
+
     // now grab the file one line at a time
     unsigned char temp;
     string tempString = "";
-    while (!feof(fin)) {
+    while ( !feof(fin) ) {
       temp = fgetc(fin);
       if (temp != '\n') {
-	tempString += temp;
+        tempString += temp;
       }
       else {
-	simulationText.push_back(tempString);
-	tempString = "";
+        simulationText.push_back(tempString);
+        tempString = "";
       }
     }
     fclose(fin);
 
     if (debugFlag) {
       for (unsigned int i=0; i<simulationText.size(); i++) {
-	cout << simulationText[i] << "\n";
+        cout << simulationText[i] << "\n";
       }
     }
 
@@ -531,20 +561,21 @@ int main(
       string simulationLine = simulationText[i];
       unsigned int lineLen = simulationLine.size();
       for (unsigned int j=0; j<lineLen; j++) {
-	if (simulationLine[j] == '\"') {
-	  if (debugFlag) {
-	    printf("Replaced a double quote at location %d, lineLen = %d\n",j,lineLen);
-	    printf("Line is:\n%s\n",simulationLine.c_str());
-	  }
-	  size_t textPos = j;
-	  simulationLine.replace(textPos, 1, "\\\"", 2);
-	  j++;
-	  lineLen++;  // because the line length increases as we add more characters
-	}
+        if (simulationLine[j] == '\"') {
+          if (debugFlag) {
+            cout << "Replaced a double quote at location " << j << 
+              ", lineLen = " << lineLen << "\n";
+            cout << "Line is:\n" << simulationLine.c_str() << "\n";
+          }
+          size_t textPos = j;
+          simulationLine.replace(textPos, 1, "\\\"", 2);
+          j++;
+          lineLen++;  // because the line length increases as we add more characters
+        }
       }
       // I could check to make sure that simulationLine was altered, but
       // I can't be bothered, and it should be pretty quick anyway
-      simulationText[i] = simulationLine;  
+      simulationText[i] = simulationLine;
     }
 
     // now we need to tear the text to bits a little
@@ -568,33 +599,34 @@ int main(
     // go through the text and rip out the header, body and footer
     for (unsigned int i=0; i<simulationText.size(); i++) {
       if (simulationText[i].find(simulationStartTag) != string::npos) {
-	foundSimStartTag = true;
-	if (verbose) {
-	  printf("Found the <simulation> start tag when pulling to bits!\n");
-	}
+        foundSimStartTag = true;
+        if (verbose) {
+          printf("Found the <simulation> start tag when pulling to bits!\n");
+        }
       }
       if (simulationText[i].find(simulationEndTag) != string::npos) {
-	foundSimEndTag = true;
-	if (verbose) {
-	  printf("Found the </simulation> end tag when pulling to bits!\n");
-	}
+        foundSimEndTag = true;
+        if (verbose) {
+          printf("Found the </simulation> end tag when pulling to bits!\n");
+        }
       }
       if (!foundSimStartTag && !foundSimEndTag) {
-	simHeaderText.push_back(simulationText[i]);
+        simHeaderText.push_back(simulationText[i]);
       }
       if (foundSimStartTag && !foundSimEndTag) {
-	simBodyText.push_back(simulationText[i]);
+        simBodyText.push_back(simulationText[i]);
       }
       if (foundSimStartTag && foundSimEndTag) {
-	simFooterText.push_back(simulationText[i]);
+        simFooterText.push_back(simulationText[i]);
       }
     }
 
     // if we got to here and foundSimStart tag is still false, then barf appropriately
     if (!foundSimStartTag) {
-      printf("Failed to find the string \"<simulation>\" within the simulation text\n");
-      printf("Exiting\n");
+      cout << "Failed to find the string \"<simulation>\" within the simulation text\n";
+      cout << "Exiting\n";
       // I'm sure we should do something more intelligent here...
+      //! \todo use xmdsException objects to handle exceptions/exits properly
       return 1;
     }
 
@@ -603,7 +635,7 @@ int main(
       cout << "-----------------------------\n";
       cout << "The simulation header follows:\n";
       for (unsigned int i=0; i<simHeaderText.size(); i++) {
-	cout << simHeaderText[i] << "\n";
+        cout << simHeaderText[i] << "\n";
       }
       cout << "-----------------------------\n";
     }
@@ -613,15 +645,16 @@ int main(
       cout << "-----------------------------\n";
       cout << "The simulation body follows:\n";
       for (unsigned int i=0; i<simBodyText.size(); i++) {
-	cout << simBodyText[i] << "\n";
+        cout << simBodyText[i] << "\n";
       }
       cout << "-----------------------------\n";
     }
       
     if (!foundSimEndTag) {
-      printf("Failed to find the string \"</simulation>\" within the simulation text\n");
-      printf("Exiting\n");
+      cout << "Failed to find the string \"</simulation>\" within the simulation text\n";
+      cout << "Exiting\n";
       // I'm sure we should do something more intelligent here...
+      //! \todo use xmdsException objects to handle exceptions/exits properly
       return 1;
     }
 
@@ -630,66 +663,67 @@ int main(
       cout << "-----------------------------\n";
       cout << "The simulation footer follows:\n";
       for (unsigned int i=0; i<simFooterText.size(); i++) {
-	cout << simFooterText[i] << "\n";
+        cout << simFooterText[i] << "\n";
       }
       cout << "-----------------------------\n";
     }
 
   }
-  catch(XMLParserException XMLRoutinesErr) {
-    printf("Could not load Document\n");
-    printf("due to the following XMLParserException:\n");
-    printf("%s",XMLRoutinesErr.getError());
-    printf("Exiting.\n");
+  catch (XMLParserException XMLRoutinesErr) {
+    cout << "Could not load Document\n" << 
+      "due to the following XMLParserException:\n" << 
+      XMLRoutinesErr.getError() << 
+      "Exiting.\n";
     return 1;
   }
 
-  if(*theDocument->documentElement()->nodeName() != "simulation") {
-    printf("Error: Expecting root element in '%s' to be <simulation>\n",infilename);
-    printf("Exiting.\n");
+  if (*theDocument->documentElement()->nodeName() != "simulation") {
+    cout << "Error: Expecting root element in '" << infilename << 
+      "' to be <simulation>\n" <<
+      "Exiting.\n";
     return 1;
   }
 
   unsigned long xmdsBytePoint = myXMLParser.xmdsBytePoint();
 
   // create the xmdsSimulation
+  xmdsSimulation myxmdsSimulation(infilename, verbose, strcmp(MPICC, ""));
 
-  xmdsSimulation myxmdsSimulation(infilename,verbose,strcmp(MPICC,""));
-
-  if(verbose) {
-    printf("Processing simulation ...\n");
+  if (verbose) {
+    cout << "Processing simulation ...\n";
   }
 
   // print out some info about xmds
-  printf("This is xmds, version %s-%s\n",
-         myxmdsSimulation.parameters()->version.c_str(),
-         myxmdsSimulation.parameters()->release.c_str());
-  printf("Copyright 2000-2004 Greg Collecutt, Joseph Hope, Paul Cochrane and others\n");
-  printf("xmds is available from http://www.xmds.org\n\n");
+  cout << "This is xmds, version " <<
+         myxmdsSimulation.parameters()->version.c_str() << "-" <<
+         myxmdsSimulation.parameters()->release.c_str() << " (" << REVISION << ")" << "\n";
+  cout << "Copyright 2000-2006 Greg Collecutt, Joseph Hope " <<
+    "and the xmds-devel team\n" << 
+    "xmds is available from http://www.xmds.org\n\n";
 
   try {
     myxmdsSimulation.processElement(theDocument->documentElement());
   }
   catch (xmdsException xmdsExceptionErr) {
-    printf("Error: simulation element could not be processed\n");
-    printf("due to the following xmdsException:\n");
-    printf("%s",xmdsExceptionErr.getError());
-    printf("Exiting.\n");
+    cout << "Error: simulation element could not be processed\n" << 
+      "due to the following xmdsException:\n" <<
+      xmdsExceptionErr.getError() << 
+      "Exiting.\n";
     return 1;
   }
 
-  if(verbose) {
-    printf("Writing output code ...\n");
+  if (verbose) {
+    cout << "Writing output code ...\n";
   }
 
   try {
     myxmdsSimulation.makeCode(xmdsBytePoint);
   }
   catch (xmdsException xmdsExceptionErr) {
-    printf("Error: simulation failed to write output code\n");
-    printf("due to the following xmdsException:\n");
-    printf("%s",xmdsExceptionErr.getError());
-    printf("Exiting.\n");
+    cout << "Error: simulation failed to write output code\n" << 
+      "due to the following xmdsException:\n" << 
+      xmdsExceptionErr.getError() <<
+      "Exiting.\n";
     return 1;
   }
 
@@ -700,8 +734,10 @@ int main(
   string cincludes     = XMDS_INCLUDES;
   string clibs         = XMDS_LIBS;
   string fftwlibs      = FFTW_LIBS;
+  string fftw3libs     = FFTW3_LIBS;
   string fftw_mpi_libs = FFTW_MPI_LIBS;
   string cthreadlibs   = THREADLIBS;
+  string fftw3threadlibs = FFTW3_THREADLIBS;
 
   // this is just some code to show what the defaults are
   if (verbose) {
@@ -713,8 +749,10 @@ int main(
          << "  cincludes = " << cincludes << "\n"
          << "  clibs = " << clibs << "\n"
          << "  fftwlibs = " << fftwlibs << "\n"
+         << "  fftw3libs = " << fftw3libs << "\n"
          << "  fftw_mpi_libs = " << fftw_mpi_libs << "\n"
-         << "  cthreadlibs = " << cthreadlibs << "\n";
+         << "  cthreadlibs = " << cthreadlibs << "\n"
+         << "  fftw3_threadlibs = " << fftw3threadlibs << "\n";
   }
 
   // if the usePrefs flag is true then try to find the prefs file
@@ -722,7 +760,7 @@ int main(
   // we fail
   if (myxmdsSimulation.parameters()->usePrefs) {
     if (verbose) {
-      printf("Using user-defined preferences\n");
+      cout << "Using user-defined preferences\n";
     }
     // now try and open the file
     // first look in ~/.xmds/xmds.prefs
@@ -737,6 +775,7 @@ int main(
     fIn.open(homeStuff.c_str());
     if (fIn.fail()) {
       // do something
+      //! \todo fail correctly if we can't open the home.stuff file
     }
     fIn >> homeDir;
     fIn.close();
@@ -750,13 +789,13 @@ int main(
     fPrefs.open(prefsFname.c_str());
     if (!fPrefs.fail()) {
       if (verbose) {
-        printf("Prefs file found: %s\n", prefsFname.c_str());
+        cout << "Prefs file found: " << prefsFname.c_str() << "\n";
       }
       // ok, now try and parse the sucker...
       parsePrefs(fPrefs, 
                  cc, cflags, clibs, cincludes, cthreadlibs, 
                  mpicc, mpicflags, 
-                 fftwlibs, fftw_mpi_libs,
+                 fftwlibs, fftw_mpi_libs, fftw3libs, fftw3threadlibs,
                  verbose, debugFlag);
 
       fPrefs.close();
@@ -765,31 +804,33 @@ int main(
     else if (fPrefs.fail()) {
       string localDir;
       if (verbose) {
-        printf("Prefs file not found at %s\n", prefsFname.c_str());
-        printf("Trying in the local directory\n");
+        cout << "Prefs file not found at " << prefsFname.c_str() << "\n";
+        cout << "Trying in the local directory\n";
         // work out what the local directory is, and report it
         system("echo $PWD > localDir.test");
         fIn.open("localDir.test");
         if (fIn.fail()) {
           // do something
+          //! \todo fail correctly if we can't open localDir.test
         }
         fIn >> localDir;
         fIn.close();
         system("rm localDir.test");
-        printf("The local directory is %s\n", localDir.c_str());
+        cout << "The local directory is " << localDir.c_str() << "\n";
       }
       prefsFname = "xmds.prefs";
       ifstream fPrefs;  // need to define fPrefs again (local to this block)
       fPrefs.open(prefsFname.c_str());
       if (!fPrefs.fail()) {
         if (verbose) {
-	  printf("Prefs file found in local directory: %s\n", localDir.c_str());
-	}
+          cout << "Prefs file found in local directory: " << 
+            localDir.c_str() << "\n";
+        }
         // ok, now try and parse the sucker...
         parsePrefs(fPrefs, 
                    cc, cflags, clibs, cincludes, cthreadlibs, 
                    mpicc, mpicflags, 
-                   fftwlibs, fftw_mpi_libs,
+                   fftwlibs, fftw_mpi_libs, fftw3libs, fftw3threadlibs,
                    verbose, debugFlag);
 
         fPrefs.close();
@@ -798,14 +839,16 @@ int main(
       // if we get to here, and things have still failed, print a warning
       // and just use the defaults
       else if (fPrefs.fail() && verbose) {
-        printf("Warning: no preferences file found.  Using default values instead\n");
+        cout << "Warning: no preferences file found.  "
+          << "Using default values instead\n";
       }
     }
 
   }
 
   if (!myxmdsSimulation.parameters()->usePrefs && verbose) {
-    printf("Warning: User-defined preferences NOT being used.  Using defaults instead\n");
+    cout << "Warning: User-defined preferences NOT being used.  " << 
+      "Using defaults instead\n";
   }
 
 
@@ -821,87 +864,115 @@ int main(
          << "  clibs = " << clibs << "\n"
          << "  fftwlibs = " << fftwlibs << "\n"
          << "  fftw_mpi_libs = " << fftw_mpi_libs << "\n"
-         << "  cthreadlibs = " << cthreadlibs << "\n";
+         << "  fftw3libs = " << fftw3libs << "\n"
+         << "  cthreadlibs = " << cthreadlibs << "\n"
+         << "  fftw3threadlibs = " << fftw3threadlibs << "\n";
   }
 
-
   // now set up the system command to compile the simulation
   char command[1024];
 
   if (compileFlag) {
-    if(myxmdsSimulation.parameters()->usempi) { //Joe mark
-      if((myxmdsSimulation.parameters()->nThreads>1)||(myxmdsSimulation.parameters()->mpiMethod=="Scheduling")) {
-        printf("compiling for MPI parallel execution with threads...\n");
-        sprintf(command,"%s -D_REENTRANT -o %s %s.cc %s %s %s %s",
-		mpicc.c_str(),
-                myxmdsSimulation.parameters()->simulationName.c_str(),
-                myxmdsSimulation.parameters()->simulationName.c_str(),
-		mpicflags.c_str(),
-                cincludes.c_str(),cthreadlibs.c_str(),fftw_mpi_libs.c_str());
-        printf("        %s\n",command);
-        if(system(command)) {
-          printf("compilation failed.\n");
+    if (myxmdsSimulation.parameters()->usempi) { //Joe mark
+      if ((myxmdsSimulation.parameters()->nThreads > 1) || 
+          (myxmdsSimulation.parameters()->mpiMethod == "Scheduling")) {
+        cout << "compiling for MPI parallel execution with threads...\n";
+        sprintf(command,"%s -D_REENTRANT -o %s %s.cc %s %s %s %s %s %s", 
+            mpicc.c_str(), 
+            myxmdsSimulation.parameters()->simulationName.c_str(), 
+            myxmdsSimulation.parameters()->simulationName.c_str(), 
+            mpicflags.c_str(), 
+            cincludes.c_str(), clibs.c_str(), 
+            myxmdsSimulation.parameters()->fftwVersion==2 ? cthreadlibs.c_str() : fftw3threadlibs.c_str(), 
+            myxmdsSimulation.parameters()->fftwVersion==2 ? fftw_mpi_libs.c_str() : fftw3libs.c_str(),
+						myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
+        cout << "        " << command << "\n";
+        if (system(command)) {
+          cout << "compilation failed.\n";
           return 1;
         }
       }
-      else if(!(myxmdsSimulation.parameters()->stochastic)){
-        printf("compiling for MPI parallel execution for a nondeterministic simulation...\n");
-        sprintf(command,"%s -o %s %s.cc %s %s %s",
-		mpicc.c_str(),
-                myxmdsSimulation.parameters()->simulationName.c_str(),
-                myxmdsSimulation.parameters()->simulationName.c_str(),
-		mpicflags.c_str(),
-                cincludes.c_str(),fftw_mpi_libs.c_str());
-        printf("        %s\n",command);
-        if(system(command)) {
-          printf("compilation failed.\n");
+      else if (!(myxmdsSimulation.parameters()->stochastic)) {
+        cout << "compiling for MPI parallel execution for a "
+          "nondeterministic simulation...\n";
+        sprintf(command,"%s -o %s %s.cc %s %s %s %s %s", 
+            mpicc.c_str(), 
+            myxmdsSimulation.parameters()->simulationName.c_str(),
+            myxmdsSimulation.parameters()->simulationName.c_str(),
+            mpicflags.c_str(), clibs.c_str(),
+            cincludes.c_str(), myxmdsSimulation.parameters()->fftwVersion==2 ? fftw_mpi_libs.c_str() : fftw3libs.c_str(),
+						myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
+        cout << "        " << command << "\n";
+        if (system(command)) {
+          cout << "compilation failed.\n";
           return 1;
         }
       }
       else {
         printf("compiling for MPI parallel execution ...\n");
-        sprintf(command,"%s -o %s %s.cc %s %s %s",
-		mpicc.c_str(),
-                myxmdsSimulation.parameters()->simulationName.c_str(),
-                myxmdsSimulation.parameters()->simulationName.c_str(),
-		mpicflags.c_str(),
-                cincludes.c_str(),fftw_mpi_libs.c_str());
-        printf("        %s\n",command);
-        if(system(command)) {
-          printf("compilation failed.\n");
+        sprintf(command,"%s -o %s %s.cc %s %s %s %s %s", 
+            mpicc.c_str(),
+            myxmdsSimulation.parameters()->simulationName.c_str(),
+            myxmdsSimulation.parameters()->simulationName.c_str(),
+            mpicflags.c_str(), clibs.c_str(),
+            cincludes.c_str(),
+            myxmdsSimulation.parameters()->fftwVersion==2 ? fftw_mpi_libs.c_str() : fftw3libs.c_str(),
+						myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
+        cout << "        " << command << "\n";
+        if (system(command)) {
+          cout << "compilation failed.\n";
           return 1;
         }
       }
     }
     else {
-      if(myxmdsSimulation.parameters()->nThreads>1) {
-        printf("compiling for threaded parallel execution...\n");
-        sprintf(command,"%s -D_REENTRANT %s -o %s %s.cc %s %s %s",
-		cc.c_str(),cflags.c_str(),
-                myxmdsSimulation.parameters()->simulationName.c_str(),
-                myxmdsSimulation.parameters()->simulationName.c_str(),
-                cincludes.c_str(),cthreadlibs.c_str(),fftwlibs.c_str());
-        printf("        %s\n",command);
-        if(system(command)) {
-          printf("compilation failed.\n");
+      if (myxmdsSimulation.parameters()->nThreads > 1) {
+        cout << "compiling for threaded parallel execution...\n";
+        sprintf(command,"%s -D_REENTRANT %s -o %s %s.cc %s %s %s %s %s %s",
+            cc.c_str(),cflags.c_str(),
+            myxmdsSimulation.parameters()->simulationName.c_str(),
+            myxmdsSimulation.parameters()->simulationName.c_str(),
+            cincludes.c_str(), clibs.c_str(),
+            myxmdsSimulation.parameters()->fftwVersion == 2? cthreadlibs.c_str() : fftw3threadlibs.c_str(),
+            myxmdsSimulation.parameters()->fftwVersion==2? fftwlibs.c_str() : fftw3libs.c_str(),
+            myxmdsSimulation.parameters()->useOpenMP ? "-openmp" : "",
+						myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
+        cout << "        " << command << "\n";
+        if (system(command)) {
+          cout << "compilation failed.\n";
           return 1;
         }
       }
       else {
-        printf("compiling ...\n");
-        sprintf(command,"%s %s -o %s %s.cc %s %s %s",
-		cc.c_str(),cflags.c_str(),
-                myxmdsSimulation.parameters()->simulationName.c_str(),
-                myxmdsSimulation.parameters()->simulationName.c_str(),
-                cincludes.c_str(),clibs.c_str(),fftwlibs.c_str());
-        printf("        %s\n",command);
-        if(system(command)) {
-          printf("compilation failed.\n");
+        cout << "compiling ...\n";
+        sprintf(command,"%s %s -o %s %s.cc %s %s %s %s",
+            cc.c_str(),cflags.c_str(),
+            myxmdsSimulation.parameters()->simulationName.c_str(),
+            myxmdsSimulation.parameters()->simulationName.c_str(),
+            cincludes.c_str(), clibs.c_str(),
+            myxmdsSimulation.parameters()->fftwVersion==2? fftwlibs.c_str() : fftw3libs.c_str(),
+						myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
+        cout << "        " << command << "\n";
+        if (system(command)) {
+          cout << "compilation failed.\n";
           return 1;
         }
       }
     }
-    printf("\n%s ready to execute\n",myxmdsSimulation.parameters()->simulationName.c_str());
+
+    // ok, we're ready to go; tell the user about it
+    cout << "\n" << myxmdsSimulation.parameters()->simulationName.c_str() << 
+      " ready to execute\n";
   }
   return 0;
 }
+
+/*
+ * Local variables:
+ * c-indentation-style: bsd
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * End:
+ *
+ * vim: tabstop=2 expandtab shiftwidth=2:
+ */
