@@ -1,5 +1,5 @@
-/*
-  Copyright (C) 2000-2004
+ /*
+  Copyright (C) 2000-2006
 
   Code contributed by Greg Collecutt, Joseph Hope and Paul Cochrane
 
@@ -21,7 +21,7 @@
 */
 
 /*
-  $Id: xmdsclasses.h,v 1.39 2005/10/18 23:33:16 joehope Exp $
+  $Id: xmdsclasses.h 1353 2007-06-07 05:51:02Z grahamdennis $
 */
 
 /*! @file xmdsclasses.h
@@ -209,6 +209,11 @@ enum xmdsVectorType {
   COMPLEX       = 0,
   DOUBLE        = 1};
 
+enum xmdsIsLoopParallelisable {
+	PARALLELISE_LOOP = true,
+	DO_NOT_PARALLELISE_LOOP = false
+};
+
 class xmdsVector;
 
 class xmdsVectorElement;
@@ -285,7 +290,9 @@ public :
   void openLoops(
                  FILE *const outfile,
                  unsigned long space,
-                 const list<XMLString>& vectorNamesList) const;
+                 const list<XMLString>& vectorNamesList,
+				 xmdsIsLoopParallelisable isLoopParallelisable = DO_NOT_PARALLELISE_LOOP,
+				 const list<XMLString>& threadPrivateVariables = list<XMLString>()) const;
 
   //! Closes the loops
   void closeLoops(
@@ -480,6 +487,8 @@ private :
   int           myInputFileFormat;     //!< The input file format
   list<XMLString> myVectorNamesList;   //!< The list of vector names
   XMLString     myCode;                //!< The string of code for initialisation of the vector(s)
+  string		myInitialisationMomentGroupName;	//!< The moment group number in the XSIL file that will be used for initialisation
+  bool			isGeometryMatchingModeStrict;		//!< True if the XSIL input geometry must exactly match the field geometry
 
   //! Write the initialisation routine to file
   void writeInitialiseRoutine(
@@ -669,6 +678,9 @@ public :
   //! Function to determine if simulation requires integrations
   bool requiresIntegrations() const;
 
+  //! Function to determine if moment group has post-propogation ,and should force initialisation
+  bool requiresInitialisation() const;
+
   //! Function to determine if simulation requires a complex raw vector
   bool needscomplexRawVector() const;
 
@@ -691,6 +703,7 @@ private :
   xmdsFieldGeometry     tempGeometry;                    //!< Temporary field geometry
   bool                  complexRawVector;                //!< Whether or not we have a complex raw vector
   bool                  myRequiresIntegrations;          //!< Whether or not integrations are required
+  bool                  myRequiresInitialisation;        //!< Whether or not initialisation is required
 
   mutable unsigned long nSamples;                        //!< The number of samples to take
 
@@ -727,79 +740,6 @@ private :
                              const unsigned long& index) const;
 };
 
-// *****************************************************************************
-// *****************************************************************************
-//                              xmdsBreakPoint
-// *****************************************************************************
-// *****************************************************************************
-
-//! xmds break point class
-class xmdsBreakPoint : public xmdsField {
-
-public :
-
-  //! Constructor for the xmdsBreakPoint class
-  /*! 
-    @param yourSimulation an xmdsSimulation object
-    @param yourVerboseMode a boolean
-
-    @todo Break points need to be implemented
-  */
-  xmdsBreakPoint(
-		 const xmdsSimulation *const yourSimulation,
-		 const bool& yourVerboseMode);
-
-  //! Destructor
-  ~xmdsBreakPoint();
-
-  //! Process elements within a moment group
-  void processElement(
-                      const Element *const yourElement);
-
-private :
-
-  XMLString             bpFname;  //!< The filename for the breakpoint file
-  XMLString             bpStatus; //!< The status of the breakpoint file (old, new, unknown)
-  XMLString             bpFormat; //!< The format of the breakpoint file (ascii, binary)
-
-  /*
-    const unsigned long   myGroupNumber;
-
-    list<XMLString>       myVectorNamesList;
-    unsigned long         mySamplingSpace;
-    list<unsigned long>   mySamplingLatticeList;
-    list<XMLString>       mySamplingMomentsList;
-    XMLString             mySamplingCode;
-    unsigned long         myRawSpace;
-    unsigned long         myPostSpace;
-    list<XMLString>       myPostMomentsList;
-    XMLString             myPostCode;
-    list<XMLString>       myPostProcessingVectorNamesList;
-    list<unsigned long>   myMain2PostDimList;
-    list<unsigned long>   myPost2MainDimList;
-    xmdsFieldGeometry     tempGeometry;
-    bool                  complexRawVector;
-    bool                  myRequiresIntegrations;
-  */
-
-  //! Write the breakpoint specific defines to file
-  void writeDefines(
-                    FILE *const outfile) const;
-
-  //! Write the breakpoint specific globals to file
-  void writeGlobals(
-                    FILE *const outfile) const;
-
-  //! Write the breakpoint prototypes to file
-  void writePrototypes(
-                       FILE *const outfile) const;
-
-  //! Write the breakpoint routines to file
-  void writeRoutines(
-                     FILE *const outfile) const;
-
-};
-
 // *****************************************************************************
 // *****************************************************************************
 //                              xmdsOutput
@@ -877,8 +817,6 @@ private :
   //! Create an xmds moment group object
   xmdsMomentGroup* createxmdsMomentGroup();
 
-  //! Create an xmds breakpoint object
-  xmdsBreakPoint* createxmdsBreakPoint();
 };
 
 
@@ -908,8 +846,13 @@ public :
   virtual void outputSampleCount() const;
 
   //! Writes initialisation calls to file
-  virtual void writeInitialisationCalls(
-                                        FILE *const outfile) const;
+  virtual void writeInitialisationCalls(FILE *const outfile) const;
+
+  //! Writes destruction calls to file
+  virtual void writeDestructionCalls(FILE *const outfile) const;
+  
+  //! Whether or not the segment needs to run when the simulation is in overtime
+  virtual bool runsInOvertime() const = 0;
 };
 
 // *****************************************************************************
@@ -938,6 +881,9 @@ public :
   //! Returns the output sample count of the sequence
   void outputSampleCount() const;
 
+  //! Whether or not the segment needs to run when the simulation is in overtime
+  bool runsInOvertime() const;
+
 private :
 
   unsigned long nCycles;                   //!< The number of cycles in the sequence
@@ -968,12 +914,24 @@ private :
   //! Create an xmds integrate RK4 IP object
   xmdsSegment* createxmdsIntegrateRK4IP();
 
+  //! Create an xmds integrate RK9 EX object
+  xmdsSegment* createxmdsIntegrateRK9EX();
+
+  //! Create an xmds integrate RK9 IP object
+  xmdsSegment* createxmdsIntegrateRK9IP();
+
   //! Create an xmds integrate ARK45 EX object
   xmdsSegment* createxmdsIntegrateARK45EX();
 
   //! Create an xmds integrate ARK45 IP object
   xmdsSegment* createxmdsIntegrateARK45IP();
 
+  //! Create an xmds integrate ARK89 EX object
+  xmdsSegment* createxmdsIntegrateARK89EX();
+
+  //! Create an xmds integrate ARK89 IP object
+  xmdsSegment* createxmdsIntegrateARK89IP();
+
   //! Create an xmds integrate SI EX object
   xmdsSegment* createxmdsIntegrateSIEX();
 
@@ -982,6 +940,9 @@ private :
 
   //! Create an xmds filte object
   xmdsSegment* createxmdsFilter();
+  
+  //! Create an xmds breakpoint object
+  xmdsSegment* createxmdsBreakpoint();
 };
 
 //! Structure containing components of an xmdsSequence
@@ -1022,7 +983,7 @@ public :
   //! Constructor for xmdsIntegrate object
   xmdsIntegrate(
                 const xmdsSimulation *const yourSimulation,
-                const bool& yourVerboseMode, const bool& adaptiveIP = false);
+                const bool& yourVerboseMode, const bool& adaptiveIP, const bool& is9thOrderIP);
 
   //! Destructor
   ~xmdsIntegrate();
@@ -1031,6 +992,9 @@ public :
   void processElement(
                       const Element *const yourElement);
 
+  //! Whether or not the segment needs to run when the simulation is in overtime
+  bool runsInOvertime() const;
+
 protected :
 
   //! Writes the defines to outfile
@@ -1118,6 +1082,9 @@ protected :
   //! returns true if the algorithm is ARK45IP 
   const bool AdaptiveIP() const;
   
+  //! returns true if the algorithm is RK9 or ARK89 
+  const bool Is9thOrderIP() const;
+  
   //! returns if k_propagate should always use the time-dependend version
   const bool Smallmemory() const;
 
@@ -1180,7 +1147,7 @@ protected :
   //! Determines if generation of noises is to be supressed
   bool noNoises() const;
 
-
+  void writeRuntimeCheckingCode(FILE *outfile) const;
 
 private :
 
@@ -1203,16 +1170,20 @@ private :
   XMLString                   myCrossPropagationCode;         //!< The cross-propagation code
   list<coStruct>              mycoStructList;                 //!< The component structure list
   unsigned long               myNextcoKey;                    //!< The key of the next component
-  bool						 myNonoises;					  //!< Flag for turning off noise generation
-  bool						  myAdaptiveIP;						// Is it adaptive?                                     
+  bool						  myNonoises;					  //!< Flag for turning off noise generation
+  bool						  myAdaptiveIP;						// Is it adaptive?       
+  bool						  myIs9thOrderIP;						// Is it 9th order?                                   
   bool                        mySmallmemory;                  // can it save six copies of k-op fields
 
   //! The output sample count 
   void outputSampleCount() const;
 
   //! Write the integrate initialisation calls to file
-  void writeInitialisationCalls(
-                                FILE *const outfile) const;
+  void writeInitialisationCalls(FILE *const outfile) const;
+
+  //! Writes destruction calls to file
+  virtual void writeDestructionCalls(FILE *const outfile) const;
+
 };
 
 // *****************************************************************************
@@ -1238,6 +1209,9 @@ public :
   void processElement(
                       const Element *const yourElement);
 
+  //! Whether or not the segment needs to run when the simulation is in overtime
+  bool runsInOvertime() const;
+
 protected:
 
   //! Returns the code element list
@@ -1290,6 +1264,56 @@ private :
              const long unsigned int& index) const;
 };
 
+// *****************************************************************************
+// *****************************************************************************
+//                              xmdsBreakpoint
+// *****************************************************************************
+// *****************************************************************************
+
+//! xmds break point class
+class xmdsBreakpoint : public xmdsSegment {
+
+public :
+
+  //! Constructor for the xmdsBreakpoint class
+  /*! 
+    @param yourSimulation an xmdsSimulation object
+    @param yourVerboseMode a boolean
+
+    @todo Break points need to be implemented
+  */
+  xmdsBreakpoint(
+		 const xmdsSimulation *const yourSimulation,
+		 const bool& yourVerboseMode);
+
+  //! Destructor
+  ~xmdsBreakpoint();
+
+  //! Process elements within a moment group
+  void processElement(const Element *const yourElement);
+
+  //! Whether or not the segment needs to run when the simulation is in overtime
+  bool runsInOvertime() const;
+
+private :
+
+  XMLString             breakpointFilename;            //!< The filename for the breakpoint file
+  unsigned long         outputSpace;                 //!< The space the output will be written in (x-space or k-space)
+  list<XMLString>       myVectorNamesList;             //!< The list of vector names
+
+  //! Write the filter prototypes to file
+  void writePrototypes(
+                       FILE *const outfile) const;
+
+  //! Write the filter routines to file
+  void writeRoutines(
+                     FILE *const outfile) const;
+
+  //! Determine the space at index (buh?)
+  bool space(
+             const long unsigned int& index) const;
+};
+
 // *****************************************************************************
 // *****************************************************************************
 //                              xmdsIntegrateIP
@@ -1348,6 +1372,19 @@ private :
   //! Write the constant k-operators propagate routine for the adaptive algorithm to file
   void writeARK45kPropagateRoutine(
                                    FILE *const outfile) const;
+								   
+  //! Write the calculate k-operator field routine for the adaptive algorithm to file
+  void writeRK9CalculatekOperatorFieldRoutine(
+                                           FILE *const outfile) const;
+                                           
+  //! Write the calculate k-operator field routine for the adaptive algorithm to file
+  void writeRK9TimeDepkCalculatekOperatorFieldRoutine(
+                                           FILE *const outfile) const;
+
+  //! Write the constant k-operators propagate routine for the adaptive algorithm to file
+  void writeRK9kPropagateRoutine (
+                                   FILE *const outfile) const;
+
 
 
 };
@@ -1493,6 +1530,74 @@ private :
   //! Write the routine to calculate the cross field to file
   void writeCalculateCrossFieldRoutine(
                                        FILE *const outfile) const;
+									   
+  XMLString     myTolerance;       // error tolerance for adaptive timestep
+  XMLString     myCutoff;          // optional function size cutoff for determination of timestep error
+  unsigned long     myMaxIterations;   // optional safety-device: aborts integration when given number of iterations is reached
+                                       
+};
+
+
+// *****************************************************************************
+// *****************************************************************************
+//                              xmdsIntegrateARK89
+// *****************************************************************************
+// *****************************************************************************
+
+//! Class to integrate equations using the fourth order Runge-Kutta method
+class xmdsIntegrateARK89 : public virtual xmdsIntegrate {
+
+public :
+
+  //! Constructor of xmdsIntegrateARK89 object
+  xmdsIntegrateARK89(
+                   const xmdsSimulation *const yourSimulation,
+                   const bool& yourVerboseMode);
+
+  //! Destructor
+  ~xmdsIntegrateARK89();
+
+  //! Processes an element
+  void processElement(
+                      const Element *const yourElement);
+
+protected :
+
+  //! returns the tolerance
+  const XMLString* tolerance() const;
+
+  //! returns the cutoff
+  const XMLString* cutoff() const;
+
+  //! Writes the function prototypes to outfile
+  void writePrototypes(
+                       FILE *const outfile) const;
+
+  //! Writes the routines to outfile
+  void writeRoutines(
+                     FILE *const outfile) const;
+
+  //! Writes the single step code to outfile
+  virtual void writeSingleStepCode(
+                                   FILE *const outfile,
+                                   const stepCaseEnum& stepCase) const = 0;
+private :
+
+  //! Write the error-finding routine to file
+  void writeTimestepErrorRoutine(
+                                 FILE *const outfile) const;
+                                 
+  //! Write the routine that finds who to sample when
+  void writeSetupSamplingRoutine(
+                                 FILE *const outfile) const;
+
+  //! Write the main integrate routine to file
+  void writeMainIntegrateRoutine(
+                                 FILE *const outfile) const;
+
+  //! Write the routine to calculate the cross field to file
+  void writeCalculateCrossFieldRoutine(
+                                       FILE *const outfile) const;								 
                                        
   XMLString     myTolerance;       // error tolerance for adaptive timestep
   XMLString     myCutoff;          // optional function size cutoff for determination of timestep error
@@ -1548,6 +1653,54 @@ private :
                                        FILE *const outfile) const;
 };
 
+// *****************************************************************************
+// *****************************************************************************
+//                              xmdsIntegrateRK9
+// *****************************************************************************
+// *****************************************************************************
+
+//! Class to integrate equations using the fourth order Runge-Kutta method
+class xmdsIntegrateRK9 : public virtual xmdsIntegrate {
+
+public :
+
+  //! Constructor of xmdsIntegrateRK9 object
+  xmdsIntegrateRK9(
+                   const xmdsSimulation *const yourSimulation,
+                   const bool& yourVerboseMode);
+
+  //! Destructor
+  ~xmdsIntegrateRK9();
+
+  //! Processes an element
+  void processElement(
+                      const Element *const yourElement);
+
+protected :
+
+  //! Writes the function prototypes to outfile
+  void writePrototypes(
+                       FILE *const outfile) const;
+
+  //! Writes the routines to outfile
+  void writeRoutines(
+                     FILE *const outfile) const;
+
+  //! Writes the single step code to outfile
+  virtual void writeSingleStepCode(
+                                   FILE *const outfile,
+                                   const stepCaseEnum& stepCase) const = 0;
+private :
+
+  //! Write the main integrate routine to file
+  void writeMainIntegrateRoutine(
+                                 FILE *const outfile) const;
+
+  //! Write the routine to calculate the cross field to file
+  void writeCalculateCrossFieldRoutine(
+                                       FILE *const outfile) const;
+};
+
 // *****************************************************************************
 // *****************************************************************************
 //                              xmdsIntegrateSIIP
@@ -1728,6 +1881,86 @@ private :
                            const stepCaseEnum& stepCase) const;
 };
 
+// *****************************************************************************
+// *****************************************************************************
+//                              xmdsIntegrateRK9IP
+// *****************************************************************************
+// *****************************************************************************
+
+//! Class to integrate equations using the fourth order Runge-Kutta method in the interaction picture
+class xmdsIntegrateRK9IP :      public xmdsIntegrateIP,
+                                public xmdsIntegrateRK9{
+
+public :
+
+  //! Constructor of xmdsIntetrageRK4IP object
+  xmdsIntegrateRK9IP(
+                     const xmdsSimulation *const yourSimulation,
+                     const bool& yourVerboseMode);
+
+  //! Destructor
+  ~xmdsIntegrateRK9IP();
+
+  //! Processes an element
+  void processElement(
+                      const Element *const yourElement);
+
+private :
+
+  //! Write the integrate RK4IP prototypes to file
+  void writePrototypes(
+                       FILE *const outfile) const;
+
+  //! Write the integrate RK4IP routines to file
+  void writeRoutines(
+                     FILE *const outfile) const;
+
+  //! Write the integrate RK4IP single step code to file
+  void writeSingleStepCode(
+                           FILE *const outfile,
+                           const stepCaseEnum& stepCase) const;
+};
+
+// *****************************************************************************
+// *****************************************************************************
+//                              xmdsIntegrateRK9EX
+// *****************************************************************************
+// *****************************************************************************
+
+//! Class to integrate equations using the fourth order Runge-Kutta method in the explicit picture
+class xmdsIntegrateRK9EX : public xmdsIntegrateEX,
+                           public xmdsIntegrateRK9 {
+
+public :
+
+  //! Constructor of an xmdsIntegrateRK9EX object
+  xmdsIntegrateRK9EX(
+                     const xmdsSimulation *const yourSimulation,
+                     const bool& yourVerboseMode);
+
+  //! Destructor
+  ~xmdsIntegrateRK9EX();
+
+  //! Processes an element
+  void processElement(
+                      const Element *const yourElement);
+
+private :
+
+  //! Write the integrate RK4EX prototypes to file
+  void writePrototypes(
+                       FILE *const outfile) const;
+
+  //! Write the integrate RK4EX routines to file
+  void writeRoutines(
+                     FILE *const outfile) const;
+
+  //! Write the integrate RK4EX single step code to file
+  void writeSingleStepCode(
+                           FILE *const outfile,
+                           const stepCaseEnum& stepCase) const;
+};
+
 // *****************************************************************************
 // *****************************************************************************
 //                              xmdsIntegrateARK45IP
@@ -1819,6 +2052,97 @@ private :
                            const stepCaseEnum& stepCase) const;
 };
 
+// *****************************************************************************
+// *****************************************************************************
+//                              xmdsIntegrateARK89IP
+// *****************************************************************************
+// *****************************************************************************
+
+//! Class to integrate equations using the adaptive fourth-fifth order Runge-Kutta method in the interaction picture
+class xmdsIntegrateARK89IP :      public xmdsIntegrateIP,
+                                public xmdsIntegrateARK89{
+
+public :
+
+  //! Constructor of xmdsIntetrageARK89IP object
+  xmdsIntegrateARK89IP(
+                     const xmdsSimulation *const yourSimulation,
+                     const bool& yourVerboseMode);
+
+  //! Destructor
+  ~xmdsIntegrateARK89IP();
+
+  //! Processes an element
+  void processElement(
+                      const Element *const yourElement);
+                    
+
+private :
+
+  //! Write the integrate ARK89IP prototypes to file
+  void writePrototypes(
+                       FILE *const outfile) const;
+                       
+  //! Writes the Routine to "undo" bad timesteps
+  void writeResetRoutine(
+					     FILE *const outfile) const;
+
+  //! Write the integrate ARK89IP routines to file
+  void writeRoutines(
+                     FILE *const outfile) const;
+                     
+
+  //! Write the integrate ARK89IP single step code to file
+  void writeSingleStepCode(
+                           FILE *const outfile,
+                           const stepCaseEnum& stepCase) const;
+                           
+};
+
+// *****************************************************************************
+// *****************************************************************************
+//                              xmdsIntegrateARK89EX
+// *****************************************************************************
+// *****************************************************************************
+
+//! Class to integrate equations using the adaptive fourth-fifth order Runge-Kutta method in the explicit picture
+class xmdsIntegrateARK89EX : public xmdsIntegrateEX,
+                           public xmdsIntegrateARK89 {
+
+public :
+
+  //! Constructor of an xmdsIntegrateARK89EX object
+  xmdsIntegrateARK89EX(
+                     const xmdsSimulation *const yourSimulation,
+                     const bool& yourVerboseMode);
+
+  //! Destructor
+  ~xmdsIntegrateARK89EX();
+
+  //! Processes an element
+  void processElement(
+                      const Element *const yourElement);
+
+private :
+
+  //! Write the integrate ARK89EX prototypes to file
+  void writePrototypes(
+                       FILE *const outfile) const;
+
+  //! Write the integrate ARK89EX routines to file
+  void writeRoutines(
+                     FILE *const outfile) const;
+
+  //! Writes the Routine to "undo" bad timesteps
+  void writeResetRoutine(
+					     FILE *const outfile) const;
+
+  //! Write the integrate ARK89EX single step code to file
+  void writeSingleStepCode(
+                           FILE *const outfile,
+                           const stepCaseEnum& stepCase) const;
+};
+
 
 // *****************************************************************************
 // *****************************************************************************
@@ -1844,7 +2168,7 @@ public :
     bool mpiAvailable;        //!< Is mpi available?
     bool usempi;              //!< Use mpi?
     bool bing;                //!< Play sound when simulation is finished?
-    long seed[2];             //!< Random number seed
+    XMLString seed[2];             //!< Random number seed
     long nNoises;             //!< Number of noise terms
     XMLString noiseKind;      //!< The kind of noise to be generated
 	XMLString mpiMethod;      //!< The method of dividing paths amongst nodes 
@@ -1855,8 +2179,13 @@ public :
     bool binaryOutput;        //!< Is binary output required?
     bool useDouble;           //!< Should double precision be used?
     bool benchmark;           //!< Time the main loop?
+    long fftwVersion;         //!< Which version of fftw shall we use?
+	  bool useOpenMP;			  		//!< Shall we use OpenMP for parallelisation?
+		bool useIntelMKL;					//!< Use Intel MKL libraries for faster random numbers
+    double runTimeLimit;      //!< Maximum running time for integrate elements
     XMLString version;        //!< The current xmds version number
     XMLString release;        //!< The current xmds release number
+    XMLString revision;       //!< The subversion revision of xmds
   };
 
   //! Structure of arguments on command line to simulation
