@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2000-2004
+ Copyright (C) 2000-2006
 
  Code contributed by Greg Collecutt, Joseph Hope and Paul Cochrane
 
@@ -21,7 +21,7 @@
 */
 
 /*
-  $Id: xmdsintegrateark45ex.cc,v 1.3 2005/05/02 02:35:58 sebwuester Exp $
+  $Id: xmdsintegrateark45ex.cc 1283 2007-01-29 21:42:00Z paultcochrane $
 */
 
 /*! @file xmdsintegrateark45ex.cc
@@ -35,21 +35,21 @@
 #include<xmdsutils.h>
 #include<xmdsclasses.h>
 
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 //                              xmdsIntegrateARK45EX public
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 
 extern bool debugFlag;
 
 long nxmdsIntegrateARK45EXs=0;  //!< The number of integrate ARK45EX objects
 
-// ******************************************************************************
+// **************************************************************************
 xmdsIntegrateARK45EX::xmdsIntegrateARK45EX(
-				       const xmdsSimulation *const yourSimulation,
-				       const bool& yourVerboseMode) :
-  xmdsIntegrate(yourSimulation,yourVerboseMode),
+               const xmdsSimulation *const yourSimulation,
+               const bool& yourVerboseMode) :
+  xmdsIntegrate(yourSimulation,yourVerboseMode,false,false),
   xmdsIntegrateEX(yourSimulation,yourVerboseMode),
   xmdsIntegrateARK45(yourSimulation,yourVerboseMode) {
   if(debugFlag) {
@@ -59,7 +59,7 @@ xmdsIntegrateARK45EX::xmdsIntegrateARK45EX(
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 xmdsIntegrateARK45EX::~xmdsIntegrateARK45EX() {
   if(debugFlag) {
     nxmdsIntegrateARK45EXs--;
@@ -68,9 +68,9 @@ xmdsIntegrateARK45EX::~xmdsIntegrateARK45EX() {
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateARK45EX::processElement(
-					const Element *const yourElement) {
+          const Element *const yourElement) {
   if(debugFlag) {
     printf("xmdsIntegrateARK45EX::processElement\n");
   }
@@ -84,15 +84,15 @@ void xmdsIntegrateARK45EX::processElement(
   xmdsIntegrateEX::processElement(yourElement);
 };
 
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 //                              xmdsIntegrateARK45EX private
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateARK45EX::writePrototypes(
-					 FILE *const outfile) const {
+           FILE *const outfile) const {
   if(debugFlag) {
     printf("xmdsIntegrateARK45EX::writePrototypes\n");
   }
@@ -124,9 +124,9 @@ void xmdsIntegrateARK45EX::writePrototypes(
   xmdsIntegrateARK45::writePrototypes(outfile);
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateARK45EX::writeRoutines(
-				       FILE *const outfile) const {
+               FILE *const outfile) const {
   if(debugFlag) {
     printf("xmdsIntegrateARK45EX::writeRoutines\n");
   }
@@ -141,14 +141,14 @@ void xmdsIntegrateARK45EX::writeRoutines(
   xmdsIntegrateARK45::writeRoutines(outfile);
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateARK45EX::writeResetRoutine(
-					     FILE *const outfile) const {
+               FILE *const outfile) const {
   if(debugFlag) {
     printf("xmdsIntegrateARK45EX::writeResetRoutine\n");
   }
 
-  const char *const fieldName = simulation()->field()->name()->c_str();	
+  const char *const fieldName = simulation()->field()->name()->c_str(); 
   
   const xmdsVector* mainVector;
 
@@ -166,22 +166,28 @@ void xmdsIntegrateARK45EX::writeResetRoutine(
   
   fprintf(outfile,"/* **************************************************/\n"); 
   fprintf(outfile,"void _segment%li_reset(%s* _reset_to,double _step){\n",segmentNumber,typeName);     
+  if (simulation()->parameters()->useOpenMP) {
+  // loop iterations are independent, so parallelise!
+      fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel for\n"
+             "#endif\n");
+  }
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"for(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",fieldName);
   }
   else {
-    fprintf(outfile,"for(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",fieldName,fieldName);
+    fprintf(outfile,"for(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",fieldName,fieldName);
   }
   fprintf(outfile,"    _%s_main[_i1] = _reset_to[_i1];\n",fieldName);
-  fprintf(outfile,"}\n");	
-  fprintf(outfile,"\n");				     
-					     
+  fprintf(outfile,"}\n"); 
+  fprintf(outfile,"\n");             
+               
 }
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateARK45EX::writeSingleStepCode(
-					     FILE *const outfile,
-					     const stepCaseEnum& stepCase) const {
+               FILE *const outfile,
+               const stepCaseEnum& stepCase) const {
   if(debugFlag) {
     printf("xmdsIntegrateARK45EX::writeSingleStepCode\n");
   }
@@ -192,9 +198,9 @@ void xmdsIntegrateARK45EX::writeSingleStepCode(
   list<XMLString> tempVectorNamesList;
   tempVectorNamesList.push_back("main");
 
-  const char* indent = "	";
+  const char* indent = "  ";
   if(simulation()->parameters()->errorCheck) {
-    indent = "		";
+    indent = "    ";
   }
 
   const char* noiseVector="";
@@ -204,27 +210,48 @@ void xmdsIntegrateARK45EX::writeSingleStepCode(
 
   simulation()->field()->vectors2space(outfile,0,tempVectorNamesList,indent);
 
+  if (simulation()->parameters()->useOpenMP) {
+   // We begin a parallel region as we have two for loops following each other
+   // This minimises the penalty of creating/destroying threads for silly implementations of OpenMP
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel\n"
+             "{\n"
+             "#pragma omp for nowait\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// a_k=y2=y1\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	akfield_%s_main[_i1] = _%s_check[_i1] = _%s_main[_i1];\n",indent,fieldName,fieldName,fieldName);
+  fprintf(outfile,"%s _akfield_%s_main[_i1] = _%s_check[_i1] = _%s_main[_i1];\n",indent,fieldName,fieldName,fieldName);
   fprintf(outfile,"\n");
 
+    if (simulation()->parameters()->useOpenMP) {
+   // Continuing the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// a_i=y1\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+  }
+  fprintf(outfile,"%s _aifield_%s_main[_i1] = _%s_main[_i1];\n",indent,fieldName,fieldName);
+  if (simulation()->parameters()->useOpenMP) {
+   // End the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "}\n"
+             "#endif\n");
   }
-  fprintf(outfile,"%s	aifield_%s_main[_i1] = _%s_main[_i1];\n",indent,fieldName,fieldName);
   fprintf(outfile,"\n");
 
-  fprintf(outfile,"%s_active_%s_main = akfield_%s_main;\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s_active_%s_main = _akfield_%s_main;\n",indent,fieldName,fieldName);
   fprintf(outfile,"\n");
 
   if(usesKOperators()) {
@@ -237,35 +264,62 @@ void xmdsIntegrateARK45EX::writeSingleStepCode(
   fprintf(outfile,"%s_segment%li_calculate_delta_a(_step,cycle%s);\n",indent,segmentNumber,noiseVector);
   fprintf(outfile,"\n");
 
+  if (simulation()->parameters()->useOpenMP) {
+   // We begin a parallel region as we have two for loops following each other
+   // This minimises the penalty of creating/destroying threads for silly implementations of OpenMP
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel\n"
+             "{\n"
+             "#pragma omp for nowait\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// y1=y1+c_1*a_k\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	_%s_main[_i1] += c[1]*akfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _%s_main[_i1] += _c[1]*_akfield_%s_main[_i1];\n",indent,fieldName,fieldName);
   
+    if (simulation()->parameters()->useOpenMP) {
+   // Continuing the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// y2=y2+cs_1*a_k\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	_%s_check[_i1] += cs[1]*akfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _%s_check[_i1] += _cs[1]*_akfield_%s_main[_i1];\n",indent,fieldName,fieldName);
 
+    if (simulation()->parameters()->useOpenMP) {
+   // Continuing the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// a_k = a_i + b_21*a_k\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+  }
+  fprintf(outfile,"%s _akfield_%s_main[_i1] = _aifield_%s_main[_i1] + _b[2][1]*_akfield_%s_main[_i1];\n",indent,fieldName,fieldName,fieldName);
+  if (simulation()->parameters()->useOpenMP) {
+   // End the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "}\n"
+             "#endif\n");
   }
-  fprintf(outfile,"%s	akfield_%s_main[_i1] = aifield_%s_main[_i1] + b[2][1]*akfield_%s_main[_i1];\n",indent,fieldName,fieldName,fieldName);
   fprintf(outfile,"\n");
 
-  fprintf(outfile,"%s%s += a[2]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s += _a[2]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
 
   if(usesKOperators()) {
@@ -278,6 +332,12 @@ void xmdsIntegrateARK45EX::writeSingleStepCode(
   fprintf(outfile,"%s_segment%li_calculate_delta_a(_step,cycle%s);\n",indent,segmentNumber,noiseVector);
   fprintf(outfile,"\n");
   
+  if (simulation()->parameters()->useOpenMP) {
+  // loop iterations are independent, so parallelise!
+      fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// c_2==cs_2==0\n",indent);
   
     fprintf(outfile,"%s// a_j = d_1*a_i + d_2*y1 + d_3*a_k \n",indent);
@@ -285,15 +345,15 @@ void xmdsIntegrateARK45EX::writeSingleStepCode(
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	ajfield_%s_main[_i1] = d[1]*aifield_%s_main[_i1] + d[2]*_%s_main[_i1]+d[3]*akfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName);
+  fprintf(outfile,"%s _ajfield_%s_main[_i1] = _d[1]*_aifield_%s_main[_i1] + _d[2]*_%s_main[_i1]+_d[3]*_akfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName);
   fprintf(outfile,"\n");
   
-  fprintf(outfile,"%s%s += a[3]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s += _a[3]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
   
-  fprintf(outfile,"%s_active_%s_main = ajfield_%s_main;\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s_active_%s_main = _ajfield_%s_main;\n",indent,fieldName,fieldName);
   fprintf(outfile,"\n");
   
   if(usesKOperators()) {
@@ -306,38 +366,65 @@ void xmdsIntegrateARK45EX::writeSingleStepCode(
   fprintf(outfile,"%s_segment%li_calculate_delta_a(_step,cycle%s);\n",indent,segmentNumber,noiseVector);
   fprintf(outfile,"\n");
 
+  if (simulation()->parameters()->useOpenMP) {
+   // We begin a parallel region as we have two for loops following each other
+   // This minimises the penalty of creating/destroying threads for silly implementations of OpenMP
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel\n"
+             "{\n"
+             "#pragma omp for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// a_l = e_1*a_i + e_2*y1 + e_3*a_k + e_4*a_j\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	alfield_%s_main[_i1] = e[1]*aifield_%s_main[_i1] + e[2]*_%s_main[_i1] + e[3]*akfield_%s_main[_i1] + e[4]*ajfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName);
+  fprintf(outfile,"%s _alfield_%s_main[_i1] = _e[1]*_aifield_%s_main[_i1] + _e[2]*_%s_main[_i1] + _e[3]*_akfield_%s_main[_i1] + _e[4]*_ajfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName);
   fprintf(outfile,"\n");
 
+    if (simulation()->parameters()->useOpenMP) {
+   // Continuing the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp for nowait\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// y1=y1+c_3*a_j\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	_%s_main[_i1] += c[3]*ajfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _%s_main[_i1] += _c[3]*_ajfield_%s_main[_i1];\n",indent,fieldName,fieldName);
   
+    if (simulation()->parameters()->useOpenMP) {
+   // Continuing the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// y2=y2+cs_3*a_j\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+  }
+  fprintf(outfile,"%s _%s_check[_i1] += _cs[3]*_ajfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  if (simulation()->parameters()->useOpenMP) {
+   // End the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "}\n"
+             "#endif\n\n");
   }
-  fprintf(outfile,"%s	_%s_check[_i1] += cs[3]*ajfield_%s_main[_i1];\n",indent,fieldName,fieldName);
   
-  fprintf(outfile,"%s%s += a[4]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s += _a[4]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
   
-  fprintf(outfile,"%s_active_%s_main = alfield_%s_main;\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s_active_%s_main = _alfield_%s_main;\n",indent,fieldName,fieldName);
   fprintf(outfile,"\n"); 
  
   if(usesKOperators()) {
@@ -350,35 +437,62 @@ void xmdsIntegrateARK45EX::writeSingleStepCode(
   fprintf(outfile,"%s_segment%li_calculate_delta_a(_step,cycle%s);\n",indent,segmentNumber,noiseVector);
   fprintf(outfile,"\n");
 
+  if (simulation()->parameters()->useOpenMP) {
+   // We begin a parallel region as we have two for loops following each other
+   // This minimises the penalty of creating/destroying threads for silly implementations of OpenMP
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel\n"
+             "{\n"
+             "#pragma omp for nowait\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// y1=y1+c_4*a_j\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	_%s_main[_i1] += c[4]*alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _%s_main[_i1] += _c[4]*_alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
   
+    if (simulation()->parameters()->useOpenMP) {
+   // Continuing the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// y2=y2+cs_4*a_j\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	_%s_check[_i1] += cs[4]*alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _%s_check[_i1] += _cs[4]*_alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
 
+    if (simulation()->parameters()->useOpenMP) {
+   // Continuing the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// a_l = f_1*a_i + f_2*y1 + f_3*a_k + f_4*a_j +f_5*a_l;\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+  }
+  fprintf(outfile,"%s _alfield_%s_main[_i1] = _f[1]*_aifield_%s_main[_i1] + _f[2]*_%s_main[_i1] + _f[3]*_akfield_%s_main[_i1] + _f[4]*_ajfield_%s_main[_i1] + _f[5]*_alfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName);
+  if (simulation()->parameters()->useOpenMP) {
+   // End the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "}\n"
+             "#endif\n\n");
   }
-  fprintf(outfile,"%s	alfield_%s_main[_i1] = f[1]*aifield_%s_main[_i1] + f[2]*_%s_main[_i1] + f[3]*akfield_%s_main[_i1] + f[4]*ajfield_%s_main[_i1] + f[5]*alfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName);
   fprintf(outfile,"\n");
 
-  fprintf(outfile,"%s%s += a[5]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s += _a[5]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
 
   if(usesKOperators()) {
@@ -391,28 +505,49 @@ void xmdsIntegrateARK45EX::writeSingleStepCode(
   fprintf(outfile,"%s_segment%li_calculate_delta_a(_step,cycle%s);\n",indent,segmentNumber,noiseVector);
   fprintf(outfile,"\n");
   
-  fprintf(outfile,"%s// c_5==0\n",indent);
   
+  if (simulation()->parameters()->useOpenMP) {
+   // We begin a parallel region as we have two for loops following each other
+   // This minimises the penalty of creating/destroying threads for silly implementations of OpenMP
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel\n"
+             "{\n"
+             "#pragma omp for\n"
+             "#endif\n");
+  }
+  fprintf(outfile,"%s// c_5==0\n",indent);
   fprintf(outfile,"%s// y2=y2+cs_5*a_l\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	_%s_check[_i1] += cs[5]*alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _%s_check[_i1] += _cs[5]*_alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
 
+    if (simulation()->parameters()->useOpenMP) {
+   // Continuing the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// a_l= g_1*a_i + g_2*a_k + g_3*a_j + g_4*y_1 +g_5*a_l +g_6*y_2;\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+  }
+  fprintf(outfile,"%s _alfield_%s_main[_i1] = _g[1]*_aifield_%s_main[_i1] + _g[2]*_akfield_%s_main[_i1] + _g[3]*_ajfield_%s_main[_i1] + _g[4]*_%s_main[_i1] + _g[5]*_alfield_%s_main[_i1] + _g[6]*_%s_check[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName);
+  if (simulation()->parameters()->useOpenMP) {
+   // End the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "}\n"
+             "#endif\n\n");
   }
-  fprintf(outfile,"%s	alfield_%s_main[_i1] = g[1]*aifield_%s_main[_i1] + g[2]*akfield_%s_main[_i1] + g[3]*ajfield_%s_main[_i1] + g[4]*_%s_main[_i1] + g[5]*alfield_%s_main[_i1] + g[6]*_%s_check[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName);
   fprintf(outfile,"\n");
 
-  fprintf(outfile,"%s%s += a[6]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s += _a[6]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
   
   if(usesKOperators()) {
@@ -425,29 +560,59 @@ void xmdsIntegrateARK45EX::writeSingleStepCode(
   fprintf(outfile,"%s_segment%li_calculate_delta_a(_step,cycle%s);\n",indent,segmentNumber,noiseVector);
   fprintf(outfile,"\n");
   
+  if (simulation()->parameters()->useOpenMP) {
+   // We begin a parallel region as we have two for loops following each other
+   // This minimises the penalty of creating/destroying threads for silly implementations of OpenMP
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel\n"
+             "{\n"
+             "#pragma omp for nowait\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// y1=y1+c_6*a_l\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	_%s_main[_i1] += c[6]*alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
-  
+  fprintf(outfile,"%s _%s_main[_i1] += _c[6]*_alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+    if (simulation()->parameters()->useOpenMP) {
+   // Continuing the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp for\n"
+             "#endif\n");
+  }
+
   fprintf(outfile,"%s// y2=y2+cs_6*a_l\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+  }
+  fprintf(outfile,"%s _%s_check[_i1] += _cs[6]*_alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  if (simulation()->parameters()->useOpenMP) {
+   // End the parallel region
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "}\n"
+             "#endif\n\n");
   }
-  fprintf(outfile,"%s	_%s_check[_i1] += cs[6]*alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
   
   fprintf(outfile,"%s// t->t+dt\n",indent); 
-  fprintf(outfile,"%s%s -= a[6]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s -= _a[6]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
   
   fprintf(outfile,"%s_active_%s_main = _%s_main;\n",indent,fieldName,fieldName);
   fprintf(outfile,"\n"); 
 };
 
+/*
+ * Local variables:
+ * c-indentation-style: bsd
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * End:
+ *
+ * vim: tabstop=2 expandtab shiftwidth=2:
+ */
