@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2000-2004
+ Copyright (C) 2000-2006
 
  Code contributed by Greg Collecutt, Joseph Hope and Paul Cochrane
 
@@ -21,11 +21,11 @@
 */
 
 /*
-  $Id: xmdsintegrateark45ip.cc,v 1.3 2005/05/02 02:35:58 sebwuester Exp $
+  $Id: xmdsintegrateark45ip.cc 1283 2007-01-29 21:42:00Z paultcochrane $
 */
 
 /*! @file xmdsintegrateark45ip.cc
-  @brief Integrate element parsing classes and methods; fourth order Runge-Kutta in the interaction picture
+  @brief Integrate element parsing classes and methods; ninth order Runge-Kutta in the interaction picture
 
   More detailed explanation...
 */
@@ -35,21 +35,21 @@
 #include<xmdsutils.h>
 #include<xmdsclasses.h>
 
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 //                              xmdsIntegrateARK45IP public
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 
 extern bool debugFlag;
 
 long nxmdsIntegrateARK45IPs=0; //!< The number of xmds integrate ARK45IP objects
 
-// ******************************************************************************
+// **************************************************************************
 xmdsIntegrateARK45IP::xmdsIntegrateARK45IP(
-				       const xmdsSimulation *const yourSimulation,
-				       const bool& yourVerboseMode) :
-  xmdsIntegrate(yourSimulation,yourVerboseMode,true),
+               const xmdsSimulation *const yourSimulation,
+               const bool& yourVerboseMode) :
+  xmdsIntegrate(yourSimulation,yourVerboseMode,true,false),
   xmdsIntegrateIP(yourSimulation,yourVerboseMode),
   xmdsIntegrateARK45(yourSimulation,yourVerboseMode) {
   if(debugFlag) {
@@ -59,7 +59,7 @@ xmdsIntegrateARK45IP::xmdsIntegrateARK45IP(
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 xmdsIntegrateARK45IP::~xmdsIntegrateARK45IP() {
   if(debugFlag) {
     nxmdsIntegrateARK45IPs--;
@@ -68,9 +68,9 @@ xmdsIntegrateARK45IP::~xmdsIntegrateARK45IP() {
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateARK45IP::processElement(
-					const Element *const yourElement) {
+          const Element *const yourElement) {
   if(debugFlag) {
     printf("xmdsIntegrateARK45IP::processElement\n");
   }
@@ -87,22 +87,22 @@ void xmdsIntegrateARK45IP::processElement(
   
 };
 
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 //                              xmdsIntegrateARK45IP protected
-// ******************************************************************************
+// **************************************************************************
 
 
 
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 //                              xmdsIntegrateARK45IP private
-// ******************************************************************************
+// **************************************************************************
 
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateARK45IP::writePrototypes(
-					 FILE *const outfile) const {
+           FILE *const outfile) const {
   if(debugFlag) {
     printf("xmdsIntegrateARK45IP::writePrototypes\n");
   }
@@ -136,9 +136,9 @@ void xmdsIntegrateARK45IP::writePrototypes(
 };
 
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateARK45IP::writeRoutines(
-				       FILE *const outfile) const {
+               FILE *const outfile) const {
   if(debugFlag) {
     printf("xmdsIntegrateARK45IP::writeRoutines\n");
   }
@@ -155,14 +155,14 @@ void xmdsIntegrateARK45IP::writeRoutines(
 
 
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateARK45IP::writeResetRoutine(
-					     FILE *const outfile) const {
+               FILE *const outfile) const {
   if(debugFlag) {
     printf("xmdsIntegrateARK45IP::writeResetRoutine\n");
   }
 
-  const char *const fieldName = simulation()->field()->name()->c_str();	
+  const char *const fieldName = simulation()->field()->name()->c_str(); 
   
   const xmdsVector* mainVector;
 
@@ -181,31 +181,37 @@ void xmdsIntegrateARK45IP::writeResetRoutine(
   fprintf(outfile,"/* **************************************************/\n"); 
   fprintf(outfile,"void _segment%li_reset(%s* _reset_to,double _step){\n",segmentNumber,typeName);
   fprintf(outfile,"\n");
+  if (simulation()->parameters()->useOpenMP) {
+  // loop iterations are independent, so parallelise!
+      fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel for\n"
+             "#endif\n");
+  }
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"for(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",fieldName);
   }
   else {
-    fprintf(outfile,"for(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",fieldName,fieldName);
+    fprintf(outfile,"for(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",fieldName,fieldName);
   }
   fprintf(outfile,"    _%s_main[_i1] = _reset_to[_i1];\n",fieldName);
-  fprintf(outfile,"\n");	
+  fprintf(outfile,"\n");  
   
   if(usesKOperators()){    
-     fprintf(outfile,"_active_%s_main=_%s_main;\n",fieldName,fieldName);	 
+     fprintf(outfile,"_active_%s_main=_%s_main;\n",fieldName,fieldName);   
      if(!Smallmemory())
        fprintf(outfile,"_segment%li_k_propagate(-1);\n",segmentNumber);
       else
        fprintf(outfile,"_segment%li_k_propagate(-1.0/5*_step);\n",segmentNumber);       
      }
-  fprintf(outfile,"}\n");	
-  fprintf(outfile,"\n");				     
-					     
+  fprintf(outfile,"}\n"); 
+  fprintf(outfile,"\n");             
+               
 }
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateARK45IP::writeSingleStepCode(
-					     FILE *const outfile,
-					     const stepCaseEnum& stepCase) const {
+               FILE *const outfile,
+               const stepCaseEnum& stepCase) const {
   if(debugFlag) {
     printf("xmdsIntegrateARK45IP::writeSingleStepCode\n");
   }
@@ -218,7 +224,7 @@ void xmdsIntegrateARK45IP::writeSingleStepCode(
   
   const unsigned long fullSpace = simulation()->field()->geometry()->fullSpace();
 
-  const char* indent = "	";
+  const char* indent = "  ";
 
   const char* noiseVector="";
   if((simulation()->parameters()->stochastic)&&(!noNoises())) {
@@ -232,14 +238,20 @@ void xmdsIntegrateARK45IP::writeSingleStepCode(
     else
      simulation()->field()->vectors2space(outfile,0,tempVectorNamesList,indent);    
 
+  if (simulation()->parameters()->useOpenMP) {
+  // loop iterations are independent, so parallelise!
+      fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// a_k=y1\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	akfield_%s_main[_i1] = _%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _akfield_%s_main[_i1] = _%s_main[_i1];\n",indent,fieldName,fieldName);
   fprintf(outfile,"\n");
   
   if(usesKOperators()) 
@@ -253,17 +265,23 @@ void xmdsIntegrateARK45IP::writeSingleStepCode(
        fprintf(outfile,"\n"); 
      }   
 
+  if (simulation()->parameters()->useOpenMP) {
+  // loop iterations are independent, so parallelise!
+      fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// a_i=y2=y1\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	aifield_%s_main[_i1] = _%s_check[_i1] = _%s_main[_i1];\n",indent,fieldName,fieldName,fieldName);
+  fprintf(outfile,"%s _aifield_%s_main[_i1] = _%s_check[_i1] = _%s_main[_i1];\n",indent,fieldName,fieldName,fieldName);
   fprintf(outfile,"\n");
 
-  fprintf(outfile,"%s_active_%s_main = akfield_%s_main;\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s_active_%s_main = _akfield_%s_main;\n",indent,fieldName,fieldName);
   fprintf(outfile,"\n");
 
   fprintf(outfile,"%s// a_k=G[a_k,t]\n",indent);
@@ -283,35 +301,30 @@ void xmdsIntegrateARK45IP::writeSingleStepCode(
        fprintf(outfile,"\n"); 
      }  
 
-  fprintf(outfile,"%s// y1=y1+c_1*a_k\n",indent);
+  if (simulation()->parameters()->useOpenMP) {
+   // We begin a parallel for loop
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel for\n"
+             "#endif\n");
+  }
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++) {\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++) {\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	_%s_main[_i1] += c[1]*akfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s// y1=y1+c_1*a_k\n",indent);
+  fprintf(outfile,"%s _%s_main[_i1] += _c[1]*_akfield_%s_main[_i1];\n",indent,fieldName,fieldName);
   
   fprintf(outfile,"%s// y2=y2+cs_1*a_k\n",indent);
-  if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
-  }
-  else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
-  }
-  fprintf(outfile,"%s	_%s_check[_i1] += cs[1]*akfield_%s_main[_i1];\n",indent,fieldName,fieldName);
-
+  fprintf(outfile,"%s _%s_check[_i1] += _cs[1]*_akfield_%s_main[_i1];\n",indent,fieldName,fieldName);
   fprintf(outfile,"%s// a_k = a_i + b_21*a_k\n",indent);
-  if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
-  }
-  else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
-  }
-  fprintf(outfile,"%s	akfield_%s_main[_i1] = aifield_%s_main[_i1] + b[2][1]*akfield_%s_main[_i1];\n",indent,fieldName,fieldName,fieldName);
+  fprintf(outfile,"%s _akfield_%s_main[_i1] = _aifield_%s_main[_i1] + _b[2][1]*_akfield_%s_main[_i1];\n",indent,fieldName,fieldName,fieldName);
+  // end for loop
+  fprintf(outfile,"%s}\n",indent);
   fprintf(outfile,"\n");
 
-  fprintf(outfile,"%s%s += a[2]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s += _a[2]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
   
   fprintf(outfile,"%s// a_k=G[a_k,t+aa_2*dt]\n",indent);
@@ -320,6 +333,12 @@ void xmdsIntegrateARK45IP::writeSingleStepCode(
      simulation()->field()->vectors2space(outfile,fullSpace,tempVectorNamesList,indent); 
   fprintf(outfile,"\n");
   
+  if (simulation()->parameters()->useOpenMP) {
+  // loop iterations are independent, so parallelise!
+      fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel for\n"
+             "#endif\n");
+  }
   fprintf(outfile,"%s// c_2==cs_2==0\n",indent);
   
     fprintf(outfile,"%s// a_j = d_1*a_i + d_2*y1 + d_3*a_k \n",indent);
@@ -327,15 +346,15 @@ void xmdsIntegrateARK45IP::writeSingleStepCode(
     fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	ajfield_%s_main[_i1] = d[1]*aifield_%s_main[_i1] + d[2]*_%s_main[_i1]+d[3]*akfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName);
+  fprintf(outfile,"%s _ajfield_%s_main[_i1] = _d[1]*_aifield_%s_main[_i1] + _d[2]*_%s_main[_i1]+_d[3]*_akfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName);
   fprintf(outfile,"\n");
   
-  fprintf(outfile,"%s%s += a[3]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s += _a[3]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
   
-  fprintf(outfile,"%s_active_%s_main = ajfield_%s_main;\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s_active_%s_main = _ajfield_%s_main;\n",indent,fieldName,fieldName);
   fprintf(outfile,"\n");
   
   if(usesKOperators()) 
@@ -366,38 +385,34 @@ void xmdsIntegrateARK45IP::writeSingleStepCode(
        fprintf(outfile,"\n"); 
     }  
 
-  fprintf(outfile,"%s// a_l = e_1*a_i + e_2*y1 + e_3*a_k + e_4*a_j\n",indent);
+  if (simulation()->parameters()->useOpenMP) {
+  // loop iterations are independent, so parallelise!
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel for\n"
+             "#endif\n");
+  }
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++) {\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++) {\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	alfield_%s_main[_i1] = e[1]*aifield_%s_main[_i1] + e[2]*_%s_main[_i1] + e[3]*akfield_%s_main[_i1] + e[4]*ajfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName);
+  fprintf(outfile,"%s// a_l = e_1*a_i + e_2*y1 + e_3*a_k + e_4*a_j\n",indent);
+  fprintf(outfile,"%s _alfield_%s_main[_i1] = _e[1]*_aifield_%s_main[_i1] + _e[2]*_%s_main[_i1] + _e[3]*_akfield_%s_main[_i1] + _e[4]*_ajfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName);
   fprintf(outfile,"\n");
 
   fprintf(outfile,"%s// y1=y1+c_3*a_j\n",indent);
-  if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
-  }
-  else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
-  }
-  fprintf(outfile,"%s	_%s_main[_i1] += c[3]*ajfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _%s_main[_i1] += _c[3]*_ajfield_%s_main[_i1];\n",indent,fieldName,fieldName);
   
   fprintf(outfile,"%s// y2=y2+cs_3*a_j\n",indent);
-  if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
-  }
-  else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
-  }
-  fprintf(outfile,"%s	_%s_check[_i1] += cs[3]*ajfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _%s_check[_i1] += _cs[3]*_ajfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  // end for loop
+  fprintf(outfile,"%s}\n", indent);
   
-  fprintf(outfile,"%s%s += a[4]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s += _a[4]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
   
-  fprintf(outfile,"%s_active_%s_main = alfield_%s_main;\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s_active_%s_main = _alfield_%s_main;\n",indent,fieldName,fieldName);
   fprintf(outfile,"\n"); 
  
   if(usesKOperators()) 
@@ -426,37 +441,33 @@ void xmdsIntegrateARK45IP::writeSingleStepCode(
        fprintf(outfile,"%s// a_l=D(-(a_4-a_2)*dt)[a_l]\n",indent);
        fprintf(outfile,"%s_segment%li_k_propagate(-2.0/5*_step);\n",indent,segmentNumber);
        fprintf(outfile,"\n"); 
-    }  
+    }
 
-  fprintf(outfile,"%s// y1=y1+c_4*a_j\n",indent);
+  if (simulation()->parameters()->useOpenMP) {
+  // loop iterations are independent, so parallelise!
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel for\n"
+             "#endif\n");
+  }
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++) {\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++) {\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	_%s_main[_i1] += c[4]*alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s// y1=y1+c_4*a_j\n",indent);
+  fprintf(outfile,"%s _%s_main[_i1] += _c[4]*_alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
   
   fprintf(outfile,"%s// y2=y2+cs_4*a_j\n",indent);
-  if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
-  }
-  else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
-  }
-  fprintf(outfile,"%s	_%s_check[_i1] += cs[4]*alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _%s_check[_i1] += _cs[4]*_alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
 
   fprintf(outfile,"%s// a_l = f_1*a_i + f_2*y1 + f_3*a_k + f_4*a_j +f_5*a_l;\n",indent);
-  if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
-  }
-  else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
-  }
-  fprintf(outfile,"%s	alfield_%s_main[_i1] = f[1]*aifield_%s_main[_i1] + f[2]*_%s_main[_i1] + f[3]*akfield_%s_main[_i1] + f[4]*ajfield_%s_main[_i1] + f[5]*alfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName);
+  fprintf(outfile,"%s _alfield_%s_main[_i1] = _f[1]*_aifield_%s_main[_i1] + _f[2]*_%s_main[_i1] + _f[3]*_akfield_%s_main[_i1] + _f[4]*_ajfield_%s_main[_i1] + _f[5]*_alfield_%s_main[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName);
+  // end for loop
+  fprintf(outfile,"%s}\n", indent);
   fprintf(outfile,"\n");
 
-  fprintf(outfile,"%s%s += a[5]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s += _a[5]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
 
   if(usesKOperators()) 
@@ -487,28 +498,30 @@ void xmdsIntegrateARK45IP::writeSingleStepCode(
        fprintf(outfile,"\n"); 
     } 
   
+  if (simulation()->parameters()->useOpenMP) {
+  // loop iterations are independent, so parallelise!
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel for\n"
+             "#endif\n");
+  }
+  if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
+    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++) {\n",indent,fieldName);
+  }
+  else {
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++) {\n",indent,fieldName,fieldName);
+  }
   fprintf(outfile,"%s// c_5==0\n",indent);
   
   fprintf(outfile,"%s// y2=y2+cs_5*a_l\n",indent);
-  if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
-  }
-  else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
-  }
-  fprintf(outfile,"%s	_%s_check[_i1] += cs[5]*alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _%s_check[_i1] += _cs[5]*_alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
 
   fprintf(outfile,"%s// a_l= g_1*a_i + g_2*a_k + g_3*a_j + g_4*y_1 +g_5*a_l +g_6*y_2;\n",indent);
-  if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
-  }
-  else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
-  }
-  fprintf(outfile,"%s	alfield_%s_main[_i1] = g[1]*aifield_%s_main[_i1] + g[2]*akfield_%s_main[_i1] + g[3]*ajfield_%s_main[_i1] + g[4]*_%s_main[_i1] + g[5]*alfield_%s_main[_i1] + g[6]*_%s_check[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName);
+  fprintf(outfile,"%s _alfield_%s_main[_i1] = _g[1]*_aifield_%s_main[_i1] + _g[2]*_akfield_%s_main[_i1] + _g[3]*_ajfield_%s_main[_i1] + _g[4]*_%s_main[_i1] + _g[5]*_alfield_%s_main[_i1] + _g[6]*_%s_check[_i1] ;\n",indent,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName,fieldName);
+  // end for loop
+  fprintf(outfile,"%s}\n", indent);
   fprintf(outfile,"\n");
 
-  fprintf(outfile,"%s%s += a[6]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s += _a[6]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
   
   if(usesKOperators()) 
@@ -539,26 +552,28 @@ void xmdsIntegrateARK45IP::writeSingleStepCode(
        fprintf(outfile,"\n"); 
     } 
   
-  fprintf(outfile,"%s// y1=y1+c_6*a_l\n",indent);
+  if (simulation()->parameters()->useOpenMP) {
+  // loop iterations are independent, so parallelise!
+    fprintf(outfile, "#ifdef _OPENMP\n"
+             "#pragma omp parallel for\n"
+             "#endif\n");
+  }
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++) {\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%sfor(long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++) {\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	_%s_main[_i1] += c[6]*alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s// y1=y1+c_6*a_l\n",indent);
+  fprintf(outfile,"%s _%s_main[_i1] += _c[6]*_alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
   
   fprintf(outfile,"%s// y2=y2+cs_6*a_l\n",indent);
-  if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%sfor(long _i1=0; _i1<total_local_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName);
-  }
-  else {
-    fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
-  }
-  fprintf(outfile,"%s	_%s_check[_i1] += cs[6]*alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _%s_check[_i1] += _cs[6]*_alfield_%s_main[_i1];\n",indent,fieldName,fieldName);
+  // end the for loop
+  fprintf(outfile,"%s}\n", indent);
   
   fprintf(outfile,"%s// t->t+dt\n",indent); 
-  fprintf(outfile,"%s%s -= a[6]*_step;\n",indent,propDim);
+  fprintf(outfile,"%s%s -= _a[6]*_step;\n",indent,propDim);
   fprintf(outfile,"\n");
 
   fprintf(outfile,"%s_active_%s_main = _%s_check;\n",indent,fieldName,fieldName);
@@ -590,3 +605,13 @@ if(usesKOperators())
     } 
   
 };
+
+/*
+ * Local variables:
+ * c-indentation-style: bsd
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * End:
+ *
+ * vim: tabstop=2 expandtab shiftwidth=2:
+ */
