@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2000-2004
+ Copyright (C) 2000-2006
 
  Code contributed by Greg Collecutt, Joseph Hope and Paul Cochrane
 
@@ -21,7 +21,7 @@
 */
 
 /*
-  $Id: xmdsintegratesiex.cc,v 1.19 2004/10/06 07:52:38 joehope Exp $
+  $Id: xmdsintegratesiex.cc 1351 2007-06-05 01:19:19Z grahamdennis $
 */
 
 /*! @file xmdsintegratesiex.cc
@@ -30,26 +30,26 @@
   More detailed explanation...
 */
 
-#include<xmlbasics.h>
-#include<dom3.h>
-#include<xmdsutils.h>
-#include<xmdsclasses.h>
+#include <xmlbasics.h>
+#include <dom3.h>
+#include <xmdsutils.h>
+#include <xmdsclasses.h>
 
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 //                              xmdsIntegrateSIEX public
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 
 extern bool debugFlag;
 
 long nxmdsIntegrateSIEXs=0;  //!< The number of xmds integrate SIEX objects
 
-// ******************************************************************************
+// **************************************************************************
 xmdsIntegrateSIEX::xmdsIntegrateSIEX(
-				     const xmdsSimulation *const yourSimulation,
-				     const bool& yourVerboseMode) :
-  xmdsIntegrate(yourSimulation,yourVerboseMode),
+             const xmdsSimulation *const yourSimulation,
+             const bool& yourVerboseMode) :
+  xmdsIntegrate(yourSimulation,yourVerboseMode,false,false),
   xmdsIntegrateEX(yourSimulation,yourVerboseMode),
   xmdsIntegrateSI(yourSimulation,yourVerboseMode) {
   if(debugFlag) {
@@ -59,7 +59,7 @@ xmdsIntegrateSIEX::xmdsIntegrateSIEX(
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 xmdsIntegrateSIEX::~xmdsIntegrateSIEX() {
   if(debugFlag) {
     nxmdsIntegrateSIEXs--;
@@ -68,9 +68,9 @@ xmdsIntegrateSIEX::~xmdsIntegrateSIEX() {
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateSIEX::processElement(
-				       const Element *const yourElement) {
+               const Element *const yourElement) {
   if(debugFlag) {
     printf("xmdsIntegrateSIEX::processElement\n");
   }
@@ -84,15 +84,15 @@ void xmdsIntegrateSIEX::processElement(
   xmdsIntegrateEX::processElement(yourElement);
 };
 
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 //                              xmdsIntegrateSIEX private
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateSIEX::writePrototypes(
-					FILE *const outfile) const {
+          FILE *const outfile) const {
   if(debugFlag) {
     printf("xmdsIntegrateSIEX::writePrototypes\n");
   }
@@ -108,14 +108,18 @@ void xmdsIntegrateSIEX::writePrototypes(
   fprintf(outfile,"\n");
 
   if(crossVectorNamesList()->size() > 0) {
-    fprintf(outfile,"void _segment%li_calculate_cross_field();\n",segmentNumber);
-    fprintf(outfile,"\n");
+	  fprintf(outfile,"void _segment%li_calculate_cross_field(",segmentNumber);
+	  if((simulation()->parameters()->stochastic)&&(!noNoises())) {
+		  fprintf(outfile,"const double *const _noise_vector");
+	  }
+	  fprintf(outfile,");\n");
+	  fprintf(outfile,"\n");
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateSIEX::writeRoutines(
-				      FILE *const outfile) const {
+              FILE *const outfile) const {
   if(debugFlag) {
     printf("xmdsIntegrateSIEX::writeRoutines\n");
   }
@@ -134,9 +138,9 @@ void xmdsIntegrateSIEX::writeRoutines(
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateSIEX::writeMainIntegrateRoutine(
-						  FILE *const outfile) const {
+              FILE *const outfile) const {
   if(debugFlag) {
     printf("xmdsIntegrateSIEX::writeMainIntegrateRoutine\n");
   }
@@ -160,7 +164,7 @@ void xmdsIntegrateSIEX::writeMainIntegrateRoutine(
   fprintf(outfile,"/* ******************************************** */\n");
   fprintf(outfile,"void _segment%li(unsigned long cycle) {\n",segmentNumber);
   fprintf(outfile,"\n");
-  fprintf(outfile,"%s *aifield_%s_main = new %s[_%s_size*_%s_main_ncomponents];\n",typeName,fieldName,typeName,fieldName,fieldName);
+  fprintf(outfile,"%s *_aifield_%s_main = (%s*)fftw_malloc(sizeof(%s)*_%s_size*_%s_main_ncomponents);\n",typeName,fieldName,typeName,typeName,fieldName,fieldName);
   fprintf(outfile,"\n");
   if((simulation()->parameters()->stochastic)&&(!noNoises())) {
     fprintf(outfile,"const double _var = 1");
@@ -177,59 +181,63 @@ void xmdsIntegrateSIEX::writeMainIntegrateRoutine(
 
   fprintf(outfile,"for(unsigned long _i0=0;_i0<%li;_i0++) {\n",lattice());
   fprintf(outfile,"\n");
+  writeRuntimeCheckingCode(outfile);
 
   if(simulation()->parameters()->errorCheck) {
-    fprintf(outfile,"	if(_half_step) {\n");
+    fprintf(outfile," if(_half_step) {\n");
     fprintf(outfile,"\n");
-    fprintf(outfile,"		const double _step = 0.5*%s/(double)%li;\n",interval()->c_str(),lattice());
+    fprintf(outfile,"   const double _step = 0.5*%s/(double)%li;\n",interval()->c_str(),lattice());
     fprintf(outfile,"\n");
     if((simulation()->parameters()->stochastic)&&(!noNoises())) {
-      fprintf(outfile,"		_make_noises(_gen1,_var/_step,_noise_vector,_%s_size*_n_noises);\n",fieldName);
+      fprintf(outfile,"   _make_noises(_gen1,_var/_step,_noise_vector,_%s_size*_n_noises);\n",fieldName);
       fprintf(outfile,"\n");
-    }		
+    }   
     writeSingleStepCode(outfile,FIRST_HALFSTEP);
     fprintf(outfile,"\n");
     if((simulation()->parameters()->stochastic)&&(!noNoises())) {
-      fprintf(outfile,"		_make_noises(_gen2,_var/_step,_noise_vector,_%s_size*_n_noises);\n",fieldName);
+      fprintf(outfile,"   _make_noises(_gen2,_var/_step,_noise_vector,_%s_size*_n_noises);\n",fieldName);
       fprintf(outfile,"\n");
-    }		
+    }   
     writeSingleStepCode(outfile,SECOND_HALFSTEP);
-    fprintf(outfile,"		}\n");
-    fprintf(outfile,"	else {\n");
+    fprintf(outfile,"   }\n");
+    fprintf(outfile," else {\n");
     fprintf(outfile,"\n");
-    fprintf(outfile,"		const double _step = %s/(double)%li;\n",interval()->c_str(),lattice());
+    fprintf(outfile,"   const double _step = %s/(double)%li;\n",interval()->c_str(),lattice());
     fprintf(outfile,"\n");
     if((simulation()->parameters()->stochastic)&&(!noNoises())) {
-      fprintf(outfile,"		_make_noises(_gen1,_var/_step/2,_noise_vector,_%s_size*_n_noises);\n",fieldName);
-      fprintf(outfile,"		_make_noises(_gen2,_var/_step/2,_noise_vector2,_%s_size*_n_noises);\n",fieldName);
-      fprintf(outfile,"		for(unsigned long _i1=0;_i1<_%s_size*_n_noises;_i1++)\n",fieldName);
-      fprintf(outfile,"			_noise_vector[_i1] += _noise_vector2[_i1];\n");
+      fprintf(outfile,"   _make_noises(_gen1,_var/_step/2,_noise_vector,_%s_size*_n_noises);\n",fieldName);
+      fprintf(outfile,"   _make_noises(_gen2,_var/_step/2,_noise_vector2,_%s_size*_n_noises);\n",fieldName);
+      fprintf(outfile,"   for(unsigned long _i1=0;_i1<_%s_size*_n_noises;_i1++)\n",fieldName);
+      fprintf(outfile,"     _noise_vector[_i1] += _noise_vector2[_i1];\n");
       fprintf(outfile,"\n");
-    }		
+    }   
     writeSingleStepCode(outfile,FULLSTEP);
-    fprintf(outfile,"		}\n");
+    fprintf(outfile,"   }\n");
   }
   else {
     fprintf(outfile,"\n");
-    fprintf(outfile,"	const double _step = %s/(double)%li;\n",interval()->c_str(),lattice());
+    fprintf(outfile," const double _step = %s/(double)%li;\n",interval()->c_str(),lattice());
     fprintf(outfile,"\n");
     if((simulation()->parameters()->stochastic)&&(!noNoises())) {
-      fprintf(outfile,"	_make_noises(_gen,_var/_step,_noise_vector,_%s_size*_n_noises);\n",fieldName);
+      fprintf(outfile," _make_noises(_gen,_var/_step,_noise_vector,_%s_size*_n_noises);\n",fieldName);
       fprintf(outfile,"\n");
-    }		
+    }   
     writeSingleStepCode(outfile,FULLSTEP);
   }
 
   for(unsigned long i=0;i<simulation()->output()->nMomentGroups();i++) {
     if(samples(i)!=0){
       fprintf(outfile,"\n");
-      fprintf(outfile,"	if(%li*((_i0+1)/%li)==(_i0+1))\n",lattice()/samples(i),lattice()/samples(i));
-      fprintf(outfile,"		_mg%li_sample();\n",i);
+      fprintf(outfile," if(%li*((_i0+1)/%li)==(_i0+1))\n",lattice()/samples(i),lattice()/samples(i));
+      fprintf(outfile,"   _mg%li_sample();\n",i);
       }
   }
-  fprintf(outfile,"	}\n");
+  fprintf(outfile," }\n");
 
-  fprintf(outfile,"    delete[] aifield_%s_main;",fieldName);
+  if (simulation()->parameters()->runTimeLimit > 0.0)
+  	fprintf(outfile, "_SEGMENT%li_END:\n", segmentNumber);
+
+  fprintf(outfile,"    fftw_free(_aifield_%s_main);",fieldName);
   if((simulation()->parameters()->stochastic)&&(!noNoises())) {
     fprintf(outfile,"    delete[] _noise_vector;\n");
     if(simulation()->parameters()->errorCheck) {
@@ -242,9 +250,9 @@ void xmdsIntegrateSIEX::writeMainIntegrateRoutine(
   fprintf(outfile,"\n");
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
-							FILE *const outfile) const {
+              FILE *const outfile) const {
   if(debugFlag) {
     printf("xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine\n");
   }
@@ -253,8 +261,16 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
   const char *const fieldName = simulation()->field()->name()->c_str();
 
   fprintf(outfile,"// *************************\n");
-  fprintf(outfile,"void _segment%li_calculate_cross_field() {",segmentNumber);
-  fprintf(outfile,"\n");
+  fprintf(outfile,"void _segment%li_calculate_cross_field(",segmentNumber);
+    if((simulation()->parameters()->stochastic)&&(!noNoises())) {
+    fprintf(outfile,"const double *const _noise_vector");
+  }
+  fprintf(outfile,") {\n");
+
+  if((simulation()->parameters()->stochastic)&&(!noNoises())) {
+    fprintf(outfile,"const double* _noises = _noise_vector;\n");
+    fprintf(outfile,"\n");
+  }
   if(crossDimNumber()+1<nDims) {
     fprintf(outfile,"const unsigned long _%s_cross_size = _%s_lattice%li",fieldName,fieldName,crossDimNumber()+1);
     for(unsigned long i=crossDimNumber()+2; i<nDims; i++) {
@@ -300,8 +316,8 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
       typeName="complex";
     }
 
-    fprintf(outfile,"%s *_%s_%s_old = new %s[_%s_cross_size*_%s_%s_ncomponents];\n",
-	    typeName,fieldName,pXMLString->c_str(),typeName,fieldName,fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s *_%s_%s_old = (%s*)fftw_malloc(sizeof(%s)*_%s_cross_size*_%s_%s_ncomponents);\n",
+      typeName,fieldName,pXMLString->c_str(),typeName,typeName,fieldName,fieldName,pXMLString->c_str());
     fprintf(outfile,"\n");
   }
 
@@ -324,16 +340,16 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
       typeName="complex";
     }
 
-    fprintf(outfile,"%s *_%s_%s_I = new %s[_%s_cross_size*_%s_%s_ncomponents];\n",
-	    typeName,fieldName,crossVector->name()->c_str(),typeName,fieldName,fieldName,crossVector->name()->c_str());
-    fprintf(outfile,"%s *_%s_%s_d = new %s[_%s_cross_size*_%s_%s_ncomponents];\n",
-	    typeName,fieldName,crossVector->name()->c_str(),typeName,fieldName,fieldName,crossVector->name()->c_str());
+    fprintf(outfile,"%s *_%s_%s_I = (%s *)fftw_malloc(sizeof(%s)*_%s_cross_size*_%s_%s_ncomponents);\n",
+      typeName,fieldName,crossVector->name()->c_str(),typeName,typeName,fieldName,fieldName,crossVector->name()->c_str());
+    fprintf(outfile,"%s *_%s_%s_d = (%s *)fftw_malloc(sizeof(%s)*_%s_cross_size*_%s_%s_ncomponents);\n",
+      typeName,fieldName,crossVector->name()->c_str(),typeName,typeName,fieldName,fieldName,crossVector->name()->c_str());
     fprintf(outfile,"\n");
 
     for(unsigned long i=0;i<crossVector->nComponents();i++) {
       fprintf(outfile,"%s d%s_d%s;\n",
-	      typeName,crossVector->componentName(i)->c_str(),
-	      simulation()->field()->geometry()->dimension(crossDimNumber())->name.c_str());
+        typeName,crossVector->componentName(i)->c_str(),
+        simulation()->field()->geometry()->dimension(crossDimNumber())->name.c_str());
     }
     fprintf(outfile,"\n");
   }
@@ -354,30 +370,30 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
   for(unsigned long i=0; i<crossDimNumber(); i++) {
 
     for(unsigned long j=0; j<i; j++) {
-	fprintf(outfile,"	");
+  fprintf(outfile," ");
     }
     fprintf(outfile,"double %s = _%s_xmin%li;\n",simulation()->field()->geometry()->dimension(i)->name.c_str(),fieldName,i);
     fprintf(outfile,"\n");
 
     for(unsigned long j=0; j<i; j++) {
-	fprintf(outfile,"	");
+  fprintf(outfile," ");
     }
     fprintf(outfile,"for(unsigned long _i%li=0; _i%li<_%s_lattice%li; _i%li++) {\n",i,i,fieldName,i,i);
     fprintf(outfile,"\n");
   }
 
   for(unsigned long j=0; j<crossDimNumber(); j++) {
-      fprintf(outfile,"	");
+      fprintf(outfile," ");
   }
   fprintf(outfile,"double %s = _%s_xmin%li;\n",
-	  simulation()->field()->geometry()->dimension(crossDimNumber())->name.c_str(),fieldName,crossDimNumber());
+    simulation()->field()->geometry()->dimension(crossDimNumber())->name.c_str(),fieldName,crossDimNumber());
   fprintf(outfile,"\n");
 
   for(unsigned long j=0; j<crossDimNumber(); j++) {
-      fprintf(outfile,"	");
+      fprintf(outfile," ");
   }
   fprintf(outfile,"for(unsigned long _i%li=0; _i%li<_%s_lattice%li-1; _i%li++) {\n",
-	  crossDimNumber(),crossDimNumber(),fieldName,crossDimNumber(),crossDimNumber());
+    crossDimNumber(),crossDimNumber(),fieldName,crossDimNumber(),crossDimNumber());
   fprintf(outfile,"\n");
 
   char indent[64];
@@ -397,7 +413,7 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
     for(unsigned long i=0; i<crossDimNumber()+1; i++) {
       fprintf(outfile,"%s_%s_%s_index_pointer_begin += _i%li",indent,fieldName,pXMLString->c_str(),i);
       for(unsigned long j=i+1;j<nDims;j++) {
-	fprintf(outfile,"*_%s_lattice%li",fieldName,j);
+  fprintf(outfile,"*_%s_lattice%li",fieldName,j);
       }
       fprintf(outfile,"*_%s_%s_ncomponents;\n",fieldName,pXMLString->c_str());
     }
@@ -408,9 +424,9 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
   fprintf(outfile,"\n");
   for(list<XMLString>::const_iterator pXMLString = crossVectorNamesList()->begin(); pXMLString != crossVectorNamesList()->end(); pXMLString++) {
     fprintf(outfile,"%sfor(unsigned long _j=0; _j<_%s_cross_size*_%s_%s_ncomponents; _j++)\n",
-	    indent,fieldName,fieldName,pXMLString->c_str());
-    fprintf(outfile,"%s	_%s_%s_I[_j] = _%s_%s[_%s_%s_index_pointer_begin + _j];\n",
-	    indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
+      indent,fieldName,fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s _%s_%s_I[_j] = _%s_%s[_%s_%s_index_pointer_begin + _j];\n",
+      indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
     fprintf(outfile,"\n");
   }
 
@@ -418,9 +434,9 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
   fprintf(outfile,"\n");
   for(list<XMLString>::const_iterator pXMLString = myMainVectorNamesList.begin(); pXMLString != myMainVectorNamesList.end(); pXMLString++) {
     fprintf(outfile,"%sfor(unsigned long _j=0; _j<_%s_cross_size*_%s_%s_ncomponents; _j++)\n",
-	    indent,fieldName,fieldName,pXMLString->c_str());
-    fprintf(outfile,"%s	_%s_%s_old[_j] = _active_%s_%s[_%s_%s_index_pointer_begin + _j];\n",
-	    indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
+      indent,fieldName,fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s _%s_%s_old[_j] = _active_%s_%s[_%s_%s_index_pointer_begin + _j];\n",
+      indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
     fprintf(outfile,"\n");
   }
 
@@ -428,31 +444,31 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
   fprintf(outfile,"\n");
   for(list<XMLString>::const_iterator pXMLString = myMainVectorNamesList.begin(); pXMLString != myMainVectorNamesList.end(); pXMLString++) {
     fprintf(outfile,"%sfor(unsigned long _j=_%s_%s_index_pointer_begin; _j < _%s_%s_index_pointer_begin + _%s_cross_size*_%s_%s_ncomponents; _j++)\n",
-	    indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,fieldName,pXMLString->c_str());
-    fprintf(outfile,"%s	_active_%s_%s[_j] = (_active_%s_%s[_j]+_active_%s_%s[_%s_cross_size*_%s_%s_ncomponents + _j])/2;\n",
-	    indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),
-	    fieldName,fieldName,pXMLString->c_str());
+      indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s _active_%s_%s[_j] = (_active_%s_%s[_j]+_active_%s_%s[_%s_cross_size*_%s_%s_ncomponents + _j])/2;\n",
+      indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),
+      fieldName,fieldName,pXMLString->c_str());
     fprintf(outfile,"\n");
   }
   fprintf(outfile,"%s// move cross dim to lattice midpoint\n",indent);
   fprintf(outfile,"\n");
   fprintf(outfile,"%s%s += _%s_dx%li/2;\n",
-	  indent,simulation()->field()->geometry()->dimension(crossDimNumber())->name.c_str(),fieldName,crossDimNumber());
+    indent,simulation()->field()->geometry()->dimension(crossDimNumber())->name.c_str(),fieldName,crossDimNumber());
   fprintf(outfile,"\n");
 
   fprintf(outfile,"%sfor(unsigned long _j=0; _j<%li; _j++) {\n",indent,nIterations());
   fprintf(outfile,"\n");
 
   for(list<XMLString>::const_iterator pXMLString = myTotalVectorsList.begin(); pXMLString != myTotalVectorsList.end(); pXMLString++) {
-    fprintf(outfile,"%s	unsigned long _%s_%s_index_pointer=_%s_%s_index_pointer_begin;\n",
-	    indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s unsigned long _%s_%s_index_pointer=_%s_%s_index_pointer_begin;\n",
+      indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
   }
   fprintf(outfile,"\n");
 
-  fprintf(outfile,"%s	// calculate delta a\n",indent);
+  fprintf(outfile,"%s // calculate delta a\n",indent);
   fprintf(outfile,"\n");
   for(list<XMLString>::const_iterator pXMLString = crossVectorNamesList()->begin(); pXMLString != crossVectorNamesList()->end(); pXMLString++) {
-    fprintf(outfile,"%s	unsigned long _%s_%s_index_pointer_local=0;\n",indent,fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s unsigned long _%s_%s_index_pointer_local=0;\n",indent,fieldName,pXMLString->c_str());
   }
   fprintf(outfile,"\n");
 
@@ -460,15 +476,15 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
   for(unsigned long i=crossDimNumber()+1; i<nDims; i++) {
 
     for(unsigned long j=0; j<i; j++) {
-	fprintf(outfile,"	");
+  fprintf(outfile," ");
     }
-    fprintf(outfile,"	double %s = _%s_xmin%li;\n",simulation()->field()->geometry()->dimension(i)->name.c_str(),fieldName,i);
+    fprintf(outfile," double %s = _%s_xmin%li;\n",simulation()->field()->geometry()->dimension(i)->name.c_str(),fieldName,i);
     fprintf(outfile,"\n");
 
     for(unsigned long j=0; j<i; j++) {
-	fprintf(outfile,"	");
+  fprintf(outfile," ");
     }
-    fprintf(outfile,"	for(unsigned long _i%li=0; _i%li<_%s_lattice%li; _i%li++) {\n",i,i,fieldName,i,i);
+    fprintf(outfile," for(unsigned long _i%li=0; _i%li<_%s_lattice%li; _i%li++) {\n",i,i,fieldName,i,i);
     fprintf(outfile,"\n");
   }
 
@@ -479,10 +495,10 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
 
   for(list<const xmdsVector*>::const_iterator pxmdsVector = crossVectorList.begin(); pxmdsVector != crossVectorList.end(); pxmdsVector++) {
     for(unsigned long i=0;i<(*pxmdsVector)->nComponents();i++) {
-      fprintf(outfile,"%s	_%s_%s_d[_%s_%s_index_pointer_local + %li] = d%s_d%s*_%s_dx%li;\n",
-	      indent2,fieldName,(*pxmdsVector)->name()->c_str(),fieldName,(*pxmdsVector)->name()->c_str(),i,
-	      (*pxmdsVector)->componentName(i)->c_str(),
-	      simulation()->field()->geometry()->dimension(crossDimNumber())->name.c_str(),fieldName,crossDimNumber());
+      fprintf(outfile,"%s _%s_%s_d[_%s_%s_index_pointer_local + %li] = d%s_d%s*_%s_dx%li;\n",
+        indent2,fieldName,(*pxmdsVector)->name()->c_str(),fieldName,(*pxmdsVector)->name()->c_str(),i,
+        (*pxmdsVector)->componentName(i)->c_str(),
+        simulation()->field()->geometry()->dimension(crossDimNumber())->name.c_str(),fieldName,crossDimNumber());
     }
     fprintf(outfile,"\n");
   }
@@ -490,56 +506,56 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
   //close inner loops
   if(crossDimNumber()+1<nDims) {
     for(list<XMLString>::const_iterator pXMLString = myTotalVectorsList.begin(); pXMLString != myTotalVectorsList.end(); pXMLString++) {
-      fprintf(outfile,"%s	_%s_%s_index_pointer += _%s_%s_ncomponents;\n",
-	      indent2,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
+      fprintf(outfile,"%s _%s_%s_index_pointer += _%s_%s_ncomponents;\n",
+        indent2,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
     }
     fprintf(outfile,"\n");
     for(list<XMLString>::const_iterator pXMLString = crossVectorNamesList()->begin(); pXMLString != crossVectorNamesList()->end(); pXMLString++) {
-      fprintf(outfile,"%s	_%s_%s_index_pointer_local += _%s_%s_ncomponents;\n",
-	      indent2,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
+      fprintf(outfile,"%s _%s_%s_index_pointer_local += _%s_%s_ncomponents;\n",
+        indent2,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
     }
     fprintf(outfile,"\n");
   }
 
   for(unsigned long i=nDims; i>crossDimNumber()+1; i--) {
     for(unsigned long j=0; j<i; j++) {
-	fprintf(outfile,"	");
+  fprintf(outfile," ");
     }
-    fprintf(outfile,"	%s += _%s_dx%li;\n",simulation()->field()->geometry()->dimension(i-1)->name.c_str(),fieldName,i-1);
+    fprintf(outfile," %s += _%s_dx%li;\n",simulation()->field()->geometry()->dimension(i-1)->name.c_str(),fieldName,i-1);
     fprintf(outfile,"\n");
     for(unsigned long j=0; j<i; j++) {
-	fprintf(outfile,"	");
+  fprintf(outfile," ");
     }
-    fprintf(outfile,"	}\n");
+    fprintf(outfile," }\n");
     fprintf(outfile,"\n");
   }
 
-  fprintf(outfile,"%s	// create next cross vectors\n",indent);
+  fprintf(outfile,"%s // create next cross vectors\n",indent);
   fprintf(outfile,"\n");
 
-  fprintf(outfile,"%s	if(_j<%li) {\n",indent,nIterations()-1);
+  fprintf(outfile,"%s if(_j<%li) {\n",indent,nIterations()-1);
   fprintf(outfile,"\n");
   for(list<XMLString>::const_iterator pXMLString = crossVectorNamesList()->begin(); pXMLString != crossVectorNamesList()->end(); pXMLString++) {
-    fprintf(outfile,"%s		for(unsigned long _k=0; _k<_%s_cross_size*_%s_%s_ncomponents; _k++)\n",
-	    indent,fieldName,fieldName,pXMLString->c_str());
-    fprintf(outfile,"%s			_%s_%s[_%s_%s_index_pointer_begin + _k] = _%s_%s_I[_k] + _%s_%s_d[_k]/2;\n",
-	    indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s   for(unsigned long _k=0; _k<_%s_cross_size*_%s_%s_ncomponents; _k++)\n",
+      indent,fieldName,fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s     _%s_%s[_%s_%s_index_pointer_begin + _k] = _%s_%s_I[_k] + _%s_%s_d[_k]/2;\n",
+      indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
     fprintf(outfile,"\n");
   }
-  fprintf(outfile,"%s		}\n",indent);
-  fprintf(outfile,"%s	else {\n",indent);
+  fprintf(outfile,"%s   }\n",indent);
+  fprintf(outfile,"%s else {\n",indent);
   fprintf(outfile,"\n");
   for(list<XMLString>::const_iterator pXMLString = crossVectorNamesList()->begin(); pXMLString != crossVectorNamesList()->end(); pXMLString++) {
-    fprintf(outfile,"%s		for(unsigned long _k=0; _k<_%s_cross_size*_%s_%s_ncomponents; _k++)\n",
-	    indent,fieldName,fieldName,pXMLString->c_str());
-    fprintf(outfile,"%s			_%s_%s[_%s_%s_index_pointer_begin + _%s_cross_size*_%s_%s_ncomponents + _k] = _%s_%s_I[_k] + _%s_%s_d[_k];\n",
-	    indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,fieldName,pXMLString->c_str(),
-	    fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s   for(unsigned long _k=0; _k<_%s_cross_size*_%s_%s_ncomponents; _k++)\n",
+      indent,fieldName,fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s     _%s_%s[_%s_%s_index_pointer_begin + _%s_cross_size*_%s_%s_ncomponents + _k] = _%s_%s_I[_k] + _%s_%s_d[_k];\n",
+      indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,fieldName,pXMLString->c_str(),
+      fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
     fprintf(outfile,"\n");
   }
-  fprintf(outfile,"%s		}\n",indent);
+  fprintf(outfile,"%s   }\n",indent);
   fprintf(outfile,"\n");
-  fprintf(outfile,"%s	}\n",indent);
+  fprintf(outfile,"%s }\n",indent);
   fprintf(outfile,"\n");
 
 
@@ -547,9 +563,9 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
   fprintf(outfile,"\n");
   for(list<XMLString>::const_iterator pXMLString = crossVectorNamesList()->begin(); pXMLString != crossVectorNamesList()->end(); pXMLString++) {
     fprintf(outfile,"%sfor(unsigned long _j=0; _j<_%s_cross_size*_%s_%s_ncomponents; _j++)\n",
-	    indent,fieldName,fieldName,pXMLString->c_str());
-    fprintf(outfile,"%s	_%s_%s[_%s_%s_index_pointer_begin + _j] = _%s_%s_I[_j];\n",
-	    indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
+      indent,fieldName,fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s _%s_%s[_%s_%s_index_pointer_begin + _j] = _%s_%s_I[_j];\n",
+      indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
     fprintf(outfile,"\n");
   }
 
@@ -557,9 +573,9 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
   fprintf(outfile,"\n");
   for(list<XMLString>::const_iterator pXMLString = myMainVectorNamesList.begin(); pXMLString != myMainVectorNamesList.end(); pXMLString++) {
     fprintf(outfile,"%sfor(unsigned long _j=0; _j<_%s_cross_size*_%s_%s_ncomponents; _j++)\n",
-	    indent,fieldName,fieldName,pXMLString->c_str());
-    fprintf(outfile,"%s	_active_%s_%s[_%s_%s_index_pointer_begin + _j] = _%s_%s_old[_j];\n",
-	    indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
+      indent,fieldName,fieldName,pXMLString->c_str());
+    fprintf(outfile,"%s _active_%s_%s[_%s_%s_index_pointer_begin + _j] = _%s_%s_old[_j];\n",
+      indent,fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str(),fieldName,pXMLString->c_str());
     fprintf(outfile,"\n");
   }
 
@@ -571,12 +587,12 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
   for(unsigned long i=crossDimNumber(); i>0; i--) {
 
     for(unsigned long j=0; j<i; j++) {
-	fprintf(outfile,"	");
+  fprintf(outfile," ");
     }
     fprintf(outfile,"%s += _%s_dx%li;\n",simulation()->field()->geometry()->dimension(i-1)->name.c_str(),fieldName,i-1);
     fprintf(outfile,"\n");
     for(unsigned long j=0; j<i; j++) {
-	fprintf(outfile,"	");
+  fprintf(outfile," ");
     }
     fprintf(outfile,"}\n");
     fprintf(outfile,"\n");
@@ -585,13 +601,13 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
   // Free allocated memory
   for(list<XMLString>::const_iterator pXMLString = myMainVectorNamesList.begin(); pXMLString != myMainVectorNamesList.end(); pXMLString++) {
 
-    fprintf(outfile,"    delete[] _%s_%s_old;\n",fieldName,pXMLString->c_str());
+    fprintf(outfile,"    fftw_free(_%s_%s_old);\n",fieldName,pXMLString->c_str());
   }
 
   for(list<XMLString>::const_iterator pXMLString = crossVectorNamesList()->begin(); pXMLString != crossVectorNamesList()->end(); pXMLString++) {
 
-    fprintf(outfile,"    delete[] _%s_%s_I;\n",fieldName,pXMLString->c_str());
-    fprintf(outfile,"    delete[] _%s_%s_d;\n",fieldName,pXMLString->c_str());
+    fprintf(outfile,"    fftw_free(_%s_%s_I);\n",fieldName,pXMLString->c_str());
+    fprintf(outfile,"    fftw_free(_%s_%s_d);\n",fieldName,pXMLString->c_str());
     fprintf(outfile,"\n");
 
   }
@@ -600,10 +616,10 @@ void xmdsIntegrateSIEX::writeCalculateCrossFieldRoutine(
 };
 
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsIntegrateSIEX::writeSingleStepCode(
-					    FILE *const outfile,
-					    const stepCaseEnum& stepCase) const {
+              FILE *const outfile,
+              const stepCaseEnum& stepCase) const {
   if(debugFlag) {
     printf("xmdsIntegrateSIEX::writeSingleStepCode\n");
   }
@@ -614,9 +630,9 @@ void xmdsIntegrateSIEX::writeSingleStepCode(
   list<XMLString> tempVectorNamesList;
   tempVectorNamesList.push_back("main");
 
-  const char* indent = "	";
+  const char* indent = "  ";
   if(simulation()->parameters()->errorCheck) {
-    indent = "		";
+    indent = "    ";
   }
 
   const char* noiseVector = "";
@@ -633,7 +649,7 @@ void xmdsIntegrateSIEX::writeSingleStepCode(
   else {
     fprintf(outfile,"%sfor(unsigned long _i1=0; _i1<_%s_size*_%s_main_ncomponents; _i1++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s	aifield_%s_main[_i1] = _%s_main[_i1];\n",indent,fieldName,fieldName);
+  fprintf(outfile,"%s _aifield_%s_main[_i1] = _%s_main[_i1];\n",indent,fieldName,fieldName);
   fprintf(outfile,"\n");
 
   fprintf(outfile,"%s%s += _step/2;\n",indent,propDim);
@@ -643,37 +659,46 @@ void xmdsIntegrateSIEX::writeSingleStepCode(
   fprintf(outfile,"\n");
 
   if(usesKOperators()) {
-    fprintf(outfile,"%s	// calculate the co derivative terms\n",indent);
-    fprintf(outfile,"%s	_segment%li_calculate_co_terms();\n",indent,segmentNumber);
+    fprintf(outfile,"%s // calculate the co derivative terms\n",indent);
+    fprintf(outfile,"%s _segment%li_calculate_co_terms();\n",indent,segmentNumber);
     fprintf(outfile,"\n");
   }
 
-  fprintf(outfile,"%s	// delta a = G[a,t]\n",indent);
-  fprintf(outfile,"%s	_segment%li_calculate_delta_a(_step,cycle%s);\n",indent,segmentNumber,noiseVector);
+  fprintf(outfile,"%s // delta a = G[a,t]\n",indent);
+  fprintf(outfile,"%s _segment%li_calculate_delta_a(_step,cycle%s);\n",indent,segmentNumber,noiseVector);
   fprintf(outfile,"\n");
 
-  fprintf(outfile,"%s	if(_i1<%li)\n",indent,nIterations()-1);
-  fprintf(outfile,"%s		// a = ai + delta a/2\n",indent);
+  fprintf(outfile,"%s if(_i1<%li)\n",indent,nIterations()-1);
+  fprintf(outfile,"%s   // a = ai + delta a/2\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%s		for(long _i2=0; _i2<total_local_size*_%s_main_ncomponents; _i2++)\n",indent,fieldName);
+    fprintf(outfile,"%s   for(long _i2=0; _i2<total_local_size*_%s_main_ncomponents; _i2++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%s		for(unsigned long _i2=0; _i2<_%s_size*_%s_main_ncomponents; _i2++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%s   for(unsigned long _i2=0; _i2<_%s_size*_%s_main_ncomponents; _i2++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s			_%s_main[_i2] = aifield_%s_main[_i2] + _%s_main[_i2]/2;\n",indent,fieldName,fieldName,fieldName);
-  fprintf(outfile,"%s	else\n",indent);
-  fprintf(outfile,"%s		// a = ai + delta a\n",indent);
+  fprintf(outfile,"%s     _%s_main[_i2] = _aifield_%s_main[_i2] + _%s_main[_i2]/2;\n",indent,fieldName,fieldName,fieldName);
+  fprintf(outfile,"%s else\n",indent);
+  fprintf(outfile,"%s   // a = ai + delta a\n",indent);
   if(simulation()->parameters()->usempi&!simulation()->parameters()->stochastic) {
-    fprintf(outfile,"%s		for(long _i2=0; _i2<total_local_size*_%s_main_ncomponents; _i2++)\n",indent,fieldName);
+    fprintf(outfile,"%s   for(long _i2=0; _i2<total_local_size*_%s_main_ncomponents; _i2++)\n",indent,fieldName);
   }
   else {
-    fprintf(outfile,"%s		for(unsigned long _i2=0; _i2<_%s_size*_%s_main_ncomponents; _i2++)\n",indent,fieldName,fieldName);
+    fprintf(outfile,"%s   for(unsigned long _i2=0; _i2<_%s_size*_%s_main_ncomponents; _i2++)\n",indent,fieldName,fieldName);
   }
-  fprintf(outfile,"%s			_%s_main[_i2] = aifield_%s_main[_i2] + _%s_main[_i2];\n",indent,fieldName,fieldName,fieldName);
-  fprintf(outfile,"%s	}\n",indent);
+  fprintf(outfile,"%s     _%s_main[_i2] = _aifield_%s_main[_i2] + _%s_main[_i2];\n",indent,fieldName,fieldName,fieldName);
+  fprintf(outfile,"%s }\n",indent);
   fprintf(outfile,"\n");
   fprintf(outfile,"%s%s += _step/2;\n",indent,propDim);
   fprintf(outfile,"\n");
 
 };
 
+/*
+ * Local variables:
+ * c-indentation-style: bsd
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * End:
+ *
+ * vim: tabstop=2 expandtab shiftwidth=2:
+ */
