@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2000-2004
+ Copyright (C) 2000-2006
 
  Code contributed by Greg Collecutt, Joseph Hope and Paul Cochrane
 
@@ -21,7 +21,7 @@
 */
 
 /*
-  $Id: xmdsutils.cc,v 1.14 2004/07/13 05:29:38 paultcochrane Exp $
+  $Id: xmdsutils.cc 1361 2007-06-21 01:31:44Z joehope $
 */
 
 /*! @file xmdsutils.cc
@@ -34,39 +34,40 @@
 #include<dom3.h>
 #include<xmdsutils.h>
 #include<string>
+#include <cstring>
 
 extern bool debugFlag;
 
 //#define DEBUG 0
 
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 //                              xmdsException
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 
-// ******************************************************************************
+// **************************************************************************
 xmdsException::xmdsException() :
   myNode(0),
   myErrorMessage("") {
 };
 
-// ******************************************************************************
+// **************************************************************************
 xmdsException::xmdsException(
-			     const char *const yourErrorMessage) :
+           const char *const yourErrorMessage) :
   myNode(0),
   myErrorMessage(yourErrorMessage) {
 };
 
-// ******************************************************************************
+// **************************************************************************
 xmdsException::xmdsException(
-			     const Node *const yourNode,
-			     const char *const yourErrorMessage) :
+           const Node *const yourNode,
+           const char *const yourErrorMessage) :
   myNode(yourNode),
   myErrorMessage(yourErrorMessage) {
 };
 
-// ******************************************************************************
+// **************************************************************************
 const char* xmdsException::getError() {
 
   char s2[256];
@@ -94,15 +95,15 @@ const char* xmdsException::getError() {
   return s;
 };
 
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 //                              xmdsUtility public
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 
 long nxmdsUtilitys=0;  //!< Number of xmds utility objects
 
-// ******************************************************************************
+// **************************************************************************
 xmdsUtility::xmdsUtility() {
   if(debugFlag) {
     nxmdsUtilitys++;
@@ -111,7 +112,7 @@ xmdsUtility::xmdsUtility() {
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 xmdsUtility::~xmdsUtility() {
   if(debugFlag) {
     nxmdsUtilitys--;
@@ -120,24 +121,24 @@ xmdsUtility::~xmdsUtility() {
   }
 };
 
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 //                              xmdsUtility protected
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 
-// ******************************************************************************
+// **************************************************************************
 char* xmdsUtility::errorMessage() const {
   return myErrorMessage;
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsUtility::getAssignmentStrings(
-				       const Element *const inElement,
-				       const XMLString& ofName,
-				       const bool& required,
-				       const unsigned long& n2get,
-				       list<XMLString>&  outList) {
+               const Element *const inElement,
+               const XMLString& ofName,
+               const bool& required,
+               const unsigned long& n2get,
+               list<XMLString>&  outList) {
   if(debugFlag) {
     printf("xmdsUtility::getAssignmentStrings\n");
   }
@@ -169,8 +170,8 @@ void xmdsUtility::getAssignmentStrings(
     if(n2get!=0) {
       // check that numbers of requested items match
       if(outList.size()!=n2get) {
-	sprintf(myErrorMessage,	"%li strings wanted but %li found",n2get,(long)outList.size());
-	throw xmdsException(nextNode,myErrorMessage);
+  sprintf(myErrorMessage, "%li strings wanted but %li found",n2get,(long)outList.size());
+  throw xmdsException(nextNode,myErrorMessage);
       }
     }
 
@@ -181,13 +182,13 @@ void xmdsUtility::getAssignmentStrings(
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsUtility::getAssignmentBools(
-				     const Element *const inElement,
-				     const XMLString& ofName,
-				     const bool& required,
-				     const unsigned long& n2get,
-				     list<bool>& outList) {
+             const Element *const inElement,
+             const XMLString& ofName,
+             const bool& required,
+             const unsigned long& n2get,
+             list<bool>& outList) {
   if(debugFlag) {
     printf("xmdsUtility::getAssignmentBools\n");
   }
@@ -220,21 +221,21 @@ void xmdsUtility::getAssignmentBools(
 
     for(list<XMLString>::const_iterator pXMLString = myOutList.begin(); pXMLString != myOutList.end(); pXMLString++) {
       if(*pXMLString=="yes") {
-	outList.push_back(1);
+  outList.push_back(1);
       }
       else if(*pXMLString=="no") {
-	outList.push_back(0);
+  outList.push_back(0);
       }
       else {
-	throw xmdsException(nextNode,"'yes' or 'no' expected");
+  throw xmdsException(nextNode,"'yes' or 'no' expected");
       }
     }
 
     if(n2get!=0) {
       // check that numbers of requested items match
       if(outList.size()!=n2get) {
-	sprintf(myErrorMessage,"%li 'yes'/'no' wanted but %li found",n2get,(long)outList.size());
-	throw xmdsException(nextNode,myErrorMessage);
+  sprintf(myErrorMessage,"%li 'yes'/'no' wanted but %li found",n2get,(long)outList.size());
+  throw xmdsException(nextNode,myErrorMessage);
       }
     }
   }
@@ -244,13 +245,13 @@ void xmdsUtility::getAssignmentBools(
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsUtility::getAssignmentULongs(
-				      const Element *const inElement,
-				      const XMLString& ofName,
-				      const bool& required,
-				      const unsigned long& n2get,
-				      list<unsigned long>& outList) {
+              const Element *const inElement,
+              const XMLString& ofName,
+              const bool& required,
+              const unsigned long& n2get,
+              list<unsigned long>& outList) {
   if(debugFlag) {
     printf("xmdsUtility::getAssignmentLongs\n");
   }
@@ -284,18 +285,18 @@ void xmdsUtility::getAssignmentULongs(
     for(list<XMLString>::const_iterator pXMLString = myOutList.begin(); pXMLString != myOutList.end(); pXMLString++) {
       unsigned long nextULong;
       if(pXMLString->asULong(nextULong)) {
-	outList.push_back(nextULong);
+  outList.push_back(nextULong);
       }
       else {
-	throw xmdsException(nextNode,"invalid positive integer");
+  throw xmdsException(nextNode,"invalid positive integer");
       }
     }
 
     if(n2get!=0) {
       // check that numbers of requested items match
       if(outList.size()!=n2get) {
-	sprintf(myErrorMessage,"%li integers wanted but %li found",n2get,(long)outList.size());
-	throw xmdsException(nextNode,myErrorMessage);
+  sprintf(myErrorMessage,"%li integers wanted but %li found",n2get,(long)outList.size());
+  throw xmdsException(nextNode,myErrorMessage);
       }
     }
   }
@@ -305,13 +306,13 @@ void xmdsUtility::getAssignmentULongs(
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsUtility::getAssignmentDoubles(
-				       const Element *const inElement,
-				       const XMLString& ofName,
-				       const bool& required,
-				       const unsigned long& n2get,
-				       list<double>& outList) {
+               const Element *const inElement,
+               const XMLString& ofName,
+               const bool& required,
+               const unsigned long& n2get,
+               list<double>& outList) {
   if(debugFlag) {
     printf("xmdsUtility::getAssignmentDoubles\n");
   }
@@ -345,18 +346,18 @@ void xmdsUtility::getAssignmentDoubles(
     for(list<XMLString>::const_iterator pXMLString = myOutList.begin(); pXMLString != myOutList.end(); pXMLString++) {
       double nextdouble;
       if(pXMLString->asDouble(nextdouble)) {
-	outList.push_back(nextdouble);
+  outList.push_back(nextdouble);
       }
       else {
-	throw xmdsException(nextNode,"invalid floating point format");
+  throw xmdsException(nextNode,"invalid floating point format");
       }
     }
 
     if(n2get!=0) {
       // check that numbers of requested items match
       if(outList.size()!=n2get) {
-	sprintf(myErrorMessage,"%li reals wanted but %li found",n2get,(long)outList.size());
-	throw xmdsException(nextNode,myErrorMessage);
+  sprintf(myErrorMessage,"%li reals wanted but %li found",n2get,(long)outList.size());
+  throw xmdsException(nextNode,myErrorMessage);
       }
     }
   }
@@ -366,13 +367,13 @@ void xmdsUtility::getAssignmentDoubles(
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsUtility::getAssignmentDomains(
-				       const Element *const inElement,
-				       const XMLString& ofName,
-				       const bool& required,
-				       const unsigned long& n2get,
-				       list<domainStruct>& outList) {
+               const Element *const inElement,
+               const XMLString& ofName,
+               const bool& required,
+               const unsigned long& n2get,
+               list<domainStruct>& outList) {
   if(debugFlag) {
     printf("xmdsUtility::getAssignmentDomains\n");
   }
@@ -406,8 +407,8 @@ void xmdsUtility::getAssignmentDomains(
     if(n2get!=0) {
       // check that numbers of requested items match
       if(myPairsList.size() != n2get) {
-	sprintf(myErrorMessage,"%li domain pairs wanted but %li found",n2get,(long)myPairsList.size());
-	throw xmdsException(nextNode,myErrorMessage);
+  sprintf(myErrorMessage,"%li domain pairs wanted but %li found",n2get,(long)myPairsList.size());
+  throw xmdsException(nextNode,myErrorMessage);
       }
     }
     for(list<XMLString>::const_iterator pXMLString = myPairsList.begin(); pXMLString != myPairsList.end(); pXMLString++) {
@@ -417,7 +418,7 @@ void xmdsUtility::getAssignmentDomains(
       parseXMLString(nextNode,&*pXMLString,myDoublesList);
 
       if(myDoublesList.size() != 2) {
-	throw xmdsException(nextNode,"bracketed pairs of reals expected");
+  throw xmdsException(nextNode,"bracketed pairs of reals expected");
       }
 
       domainStruct nextDomain;
@@ -427,16 +428,16 @@ void xmdsUtility::getAssignmentDomains(
       pXMLString2 = myDoublesList.begin();
 
       if(!pXMLString2->asDouble(nextDomain.begin)) {
-	throw xmdsException(nextNode,"invalid floating point format");
+  throw xmdsException(nextNode,"invalid floating point format");
       }
 
       pXMLString2++;
       if(!pXMLString2->asDouble(nextDomain.end)) {
-	throw xmdsException(nextNode,"invalid floating point format");
+  throw xmdsException(nextNode,"invalid floating point format");
       }
 
       if(nextDomain.end<=nextDomain.begin) {
-	throw xmdsException(nextNode,"domain ends must be > begins");
+  throw xmdsException(nextNode,"domain ends must be > begins");
       }
 
       outList.push_back(nextDomain);
@@ -449,14 +450,14 @@ void xmdsUtility::getAssignmentDomains(
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 
 void xmdsUtility::getAttributeStrings(
-				      const Element *const inElement,
-				      const XMLString& ofElementName,
-				      const XMLString& ofAttrName,
-				      const bool& required,
-				      XMLString&  outString) {
+              const Element *const inElement,
+              const XMLString& ofElementName,
+              const XMLString& ofAttrName,
+              const bool& required,
+              XMLString&  outString) {
   if(debugFlag) {
     printf("xmdsUtility::getAttributeStrings\n");
   }
@@ -491,15 +492,15 @@ void xmdsUtility::getAttributeStrings(
     if (attrNodeMap->length() > 0) {
       const Node* attrNode = attrNodeMap->getNamedItem(ofAttrName);
       if (attrNode != 0) {
-	outString = attrNode->nodeValue()->c_str();
+  outString = attrNode->nodeValue()->c_str();
       }
       // if the attribute isn't found, and it's required, barf
       else if ((attrNode == 0) && required) {
-	sprintf(myErrorMessage,"Attribute '%s' not found",ofAttrName.c_str());
-	throw xmdsException(inElement,myErrorMessage);
+  sprintf(myErrorMessage,"Attribute '%s' not found",ofAttrName.c_str());
+  throw xmdsException(inElement,myErrorMessage);
       }      
       else {
-	outString = EMPTY_STRING;
+  outString = EMPTY_STRING;
       }
     }
   }
@@ -509,13 +510,13 @@ void xmdsUtility::getAttributeStrings(
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsUtility::getAttributeBools(
-				     const Element *const inElement,
-				     const XMLString& ofElementName,
-				     const XMLString& ofAttrName,
-				     const bool& required,
-				     bool& outBool) {
+             const Element *const inElement,
+             const XMLString& ofElementName,
+             const XMLString& ofAttrName,
+             const bool& required,
+             bool& outBool) {
   if(debugFlag) {
     printf("xmdsUtility::getAttributeBools\n");
   }
@@ -548,20 +549,20 @@ void xmdsUtility::getAttributeBools(
     if (attrNodeMap->length() > 0) {
       const Node* attrNode = attrNodeMap->getNamedItem(ofAttrName);  // get the node with the name ofAttrName
       if (attrNode != 0) {
-	// then compare the value with 'yes' and 'no'
-	const string boolString = static_cast <string>(attrNode->nodeValue()->c_str());
-	if (debugFlag) {
-	  printf("xmdsUtility::getAttributeBools : boolString = %s\n", boolString.c_str());
-	}
-	if (boolString == "yes") {
-	  outBool = true;
-	}
-	else if (boolString == "no") {
-	  outBool = false;
-	}
-	else {
-	  throw xmdsException(nextNode, "'yes' or 'no' expected");
-	}
+  // then compare the value with 'yes' and 'no'
+  const string boolString = static_cast <string>(attrNode->nodeValue()->c_str());
+  if (debugFlag) {
+    printf("xmdsUtility::getAttributeBools : boolString = %s\n", boolString.c_str());
+  }
+  if (boolString == "yes") {
+    outBool = true;
+  }
+  else if (boolString == "no") {
+    outBool = false;
+  }
+  else {
+    throw xmdsException(nextNode, "'yes' or 'no' expected");
+  }
       }
     }
   }
@@ -575,9 +576,9 @@ void xmdsUtility::getAttributeBools(
 
 };
 
-// ******************************************************************************
+// **************************************************************************
 unsigned long xmdsUtility::spaceList2ULong(
-					   const list<bool>& spaceList) const {
+             const list<bool>& spaceList) const {
   if(debugFlag) {
     printf("xmdsUtility::spaceList2ULong\n");
   }
@@ -594,29 +595,29 @@ unsigned long xmdsUtility::spaceList2ULong(
   return space;
 };
 
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 //                              xmdsUtility private
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
 
-// ******************************************************************************
+// **************************************************************************
 void xmdsUtility::parseXMLString(
-				 const Node *const inNode,
-				 const XMLString *const inString,
-				 list<XMLString>& outXMLStringList) {
+         const Node *const inNode,
+         const XMLString *const inString,
+                 list<XMLString>& outXMLStringList) {
   if(debugFlag) {
     printf("xmdsUtility::parseXMLString\n");
   }
-
+  
   outXMLStringList.clear();
-
+  
   if(inString->length()==0) {
     return;
   }
-
+  
   XMLString copyInString = *inString;
-
+  
   // begin by replacing all commas and semicolons with spaces
   unsigned long i=0;
   while(i<copyInString.length()) {
@@ -625,50 +626,50 @@ void xmdsUtility::parseXMLString(
     }
     i++;
   }
-
+  
   // now mark all begins and ends
   list<unsigned long> begins;
   list<unsigned long> ends;
   long bracketLevel=0;
   i=0;
-
+  
   char lastChar=0x20;
   while(i<copyInString.length()) {
     if(copyInString.data(i)=='(') {
       if(bracketLevel==0) {
-	begins.push_back(i+1);
+        begins.push_back(i+1);
       }
       bracketLevel++;
     }
     else if(copyInString.data(i)==')') {
       bracketLevel--;
       if(bracketLevel==0) {
-	ends.push_back(i);
+        ends.push_back(i);
       }
     }
     else if(bracketLevel==0) {
-      if(!XMLChar::isWhiteSpace(copyInString.data(i))&(XMLChar::isWhiteSpace(lastChar)|(lastChar==')'))) {
-	begins.push_back(i);
+      if (!XMLChar::isWhiteSpace(copyInString.data(i)) && (XMLChar::isWhiteSpace(lastChar) || lastChar==')')) {
+        begins.push_back(i);
       }
-      else if(XMLChar::isWhiteSpace(copyInString.data(i))&!(XMLChar::isWhiteSpace(lastChar)|(lastChar==')'))) {
-	ends.push_back(i);
+      else if (XMLChar::isWhiteSpace(copyInString.data(i)) && !(XMLChar::isWhiteSpace(lastChar) || lastChar==')')) {
+        ends.push_back(i);
       }
     }
     lastChar=copyInString.data(i);
     i++;
   }
-
+  
   if(bracketLevel !=0 ) {
     throw xmdsException(inNode,"Imbalanced bracketing");
   }
-
+  
   if(!(XMLChar::isWhiteSpace(lastChar)|(lastChar==')'))) {
     ends.push_back(i);
   }
-
+  
   list<unsigned long>::const_iterator pbegin = begins.begin();
   list<unsigned long>::const_iterator pend = ends.begin();
-
+  
   while(pbegin != begins.end()) {
     XMLString subString;
     copyInString.subString(subString,*pbegin,*pend);
@@ -676,4 +677,14 @@ void xmdsUtility::parseXMLString(
     pbegin++;
     pend++;
   }
-};
+}
+
+/*
+ * Local variables:
+ * c-indentation-style: bsd
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * End:
+ *
+ * vim: tabstop=2 expandtab shiftwidth=2:
+ */
