@@ -1,7 +1,7 @@
 /*
- Copyright (C) 2000-2004
+ Copyright (C) 2000-2006
 
- Code contributed by Greg Collecutt, Joseph Hope and Paul Cochrane
+ Code contributed by Greg Collecutt, Joseph Hope, Andrew Reid and Paul Cochrane
 
  This file is part of xmds.
  
@@ -21,7 +21,7 @@
 */
 
 /*
-  $Id: xmlbasics.cc,v 1.11 2004/07/13 05:29:38 paultcochrane Exp $
+  $Id: xmlbasics.cc 1283 2007-01-29 21:42:00Z paultcochrane $
 */
 
 /*! @file xmlbasics.cc
@@ -36,15 +36,15 @@ extern bool xmlDebugFlag;
 
 //#define DEBUG 0
 
-// ******************************************************************************
-// ******************************************************************************
-//	XMLException
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
+//  XMLException
+// **************************************************************************
+// **************************************************************************
 
 long nXMLExceptions=0;  //!< The number of XML exceptions
 
-// ******************************************************************************
+// **************************************************************************
 XMLException::XMLException() {
   if(xmlDebugFlag) {
     nXMLExceptions++;
@@ -54,9 +54,9 @@ XMLException::XMLException() {
   code=UNKNOWN_ERR;
 };
 
-// ******************************************************************************
+// **************************************************************************
 XMLException::XMLException(
-			   const unsigned short& error) {
+         const unsigned short& error) {
   if(xmlDebugFlag) {
     nXMLExceptions++;
     printf("XMLException::XMLException(unsigned short error)\n");
@@ -65,7 +65,7 @@ XMLException::XMLException(
   code=error;
 };
 
-// ******************************************************************************
+// **************************************************************************
 XMLException::~XMLException() {
   if(xmlDebugFlag) {
     nXMLExceptions--;
@@ -74,7 +74,7 @@ XMLException::~XMLException() {
   }
 };
 
-// ******************************************************************************
+// **************************************************************************
 const char* XMLException::getError() const {
   switch(code) {
   case UNKNOWN_ENCODING_ERR :
@@ -94,15 +94,15 @@ const char* XMLException::getError() const {
   }
 };
 
-// ******************************************************************************
-// ******************************************************************************
-//	XMLChar
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
+//  XMLChar
+// **************************************************************************
+// **************************************************************************
 
 // **********************************************************************
 bool XMLChar::isChar(
-		     const char& ch) {
+         const char& ch) {
   if(ch>=0x0020) {
       return 1;
   }
@@ -114,7 +114,7 @@ bool XMLChar::isChar(
 
 // **********************************************************************
 bool XMLChar::isCharData(
-			 const char& ch) {
+       const char& ch) {
   if((ch=='<')|(ch=='&')) {
       return 0;
   }
@@ -123,51 +123,51 @@ bool XMLChar::isCharData(
 
 // **********************************************************************
 bool XMLChar::isWhiteSpace(
-			   const char& ch) {
+         const char& ch) {
   return ((ch==0x20)|(ch==0x09)|(ch==0x0D)|(ch==0x0A));
 };
 
 // **********************************************************************
 bool XMLChar::isLatinLetter(
-			    const char& ch) {
+          const char& ch) {
   return (((ch>='A')&(ch<='Z'))|((ch>='a')&(ch<='z')));
 };
 
 // **********************************************************************
 bool XMLChar::isLatinDigit(
-			   const char& ch) {
+         const char& ch) {
   return ((ch>='0')&(ch<='9'));
 };
 
 // **********************************************************************
 bool XMLChar::isLatinHexDigit(
-			      const char& ch) {
+            const char& ch) {
   return (((ch>='0')&(ch<='9'))
-	  |((ch>='A')&(ch<='F'))
-	  |((ch>='a')&(ch<='f')));
+    |((ch>='A')&(ch<='F'))
+    |((ch>='a')&(ch<='f')));
 };
 
 // **********************************************************************
 bool XMLChar::isLetter(
-		       const char& ch) {
+           const char& ch) {
   return isBaseChar(ch);
 };
 
 // **********************************************************************
 bool XMLChar::isBaseChar(
-			 const char& ch)	{
+       const char& ch)  {
   return isLatinLetter(ch);
 };
 
 // **********************************************************************
 bool XMLChar::isDigit(
-		      const char& ch) {
+          const char& ch) {
   return isLatinDigit(ch);
 };
 
 // **********************************************************************
 bool XMLChar::isNameChar(
-			 const char& ch) {
+       const char& ch) {
   if (isNCNameChar(ch)) {
       return 1;
   }
@@ -179,7 +179,7 @@ bool XMLChar::isNameChar(
 
 // **********************************************************************
 bool XMLChar::isNCNameChar(
-			   const char& ch) {
+         const char& ch) {
   if (isLetter(ch)) {
       return 1;
   }
@@ -194,7 +194,7 @@ bool XMLChar::isNCNameChar(
 
 // **********************************************************************
 bool XMLChar::isPubidChar(
-			  const char& ch) {
+        const char& ch) {
   if (isLatinLetter(ch)) {
       return 1;
   }
@@ -225,15 +225,15 @@ bool XMLChar::isPubidChar(
   return 0;
 };
 
-// ******************************************************************************
-// ******************************************************************************
-//	XMLString
-// ******************************************************************************
-// ******************************************************************************
+// **************************************************************************
+// **************************************************************************
+//  XMLString
+// **************************************************************************
+// **************************************************************************
 
 long nXMLStrings=0;  //!< The number of XML strings
 
-// ******************************************************************************
+// **************************************************************************
 // XMLString default constructor
 XMLString::XMLString() {
   if(xmlDebugFlag) {
@@ -248,7 +248,7 @@ XMLString::XMLString() {
 // **********************************************************************
 // XMLString construction from C-style string
 XMLString::XMLString(
-		     const char* s) {
+         const char* s) {
   if(xmlDebugFlag) {
     nXMLStrings++;
     printf("XMLString::XMLString(const char* s) = %s\n",s);
@@ -269,7 +269,7 @@ XMLString::XMLString(
 // **********************************************************************
 // XMLString construction from another XMLString
 XMLString::XMLString(
-		     const XMLString& s) {
+         const XMLString& s) {
   if(xmlDebugFlag) {
     nXMLStrings++;
     printf("XMLString::XMLString(const XMLString& s) = %s\n",s.c_str());
@@ -303,7 +303,7 @@ XMLString::~XMLString() {
 
 // **********************************************************************
 XMLString& XMLString::operator=(
-				const XMLString& s) {
+        const XMLString& s) {
   if(this != &s) { // so as to never copy itself!
     if (_length) {
       delete _data;
@@ -318,12 +318,12 @@ XMLString& XMLString::operator=(
     }
     _c_str_valid=0;
   }
-  return *this;	
+  return *this; 
 };
 
 // **********************************************************************
 XMLString& XMLString::operator=(
-				const char* s) {
+        const char* s) {
   if(_length) {
     delete _data;
   }
@@ -339,7 +339,7 @@ XMLString& XMLString::operator=(
     _data[i] = s[i];
   }
   _c_str_valid=0;
-  return *this;	
+  return *this; 
 };
 
 // **********************************************************************
@@ -358,11 +358,11 @@ const char* XMLString::c_str() const {
     _c_str_valid=1;
   }
   return _c_str;
-};	
+};  
 
 // **********************************************************************
 char XMLString::data(
-		     const unsigned long& index) const {
+         const unsigned long& index) const {
 
   if(index>=_length) {
     throw XMLException(XMLException::RANGE_ERR);
@@ -373,7 +373,7 @@ char XMLString::data(
 
 // **********************************************************************
 bool XMLString::operator==(
-			   const XMLString& s) const {
+         const XMLString& s) const {
   if(&s==this) {
     return 1;
   }
@@ -382,20 +382,20 @@ bool XMLString::operator==(
   }
   bool result=1;
   for(unsigned long i=0;i<_length;i++) {
-    result = result&(s._data[i]==_data[i]);		
+    result = result&(s._data[i]==_data[i]);   
   }
-  return result;	
+  return result;  
 };
 
 // **********************************************************************
 bool XMLString::operator!=(
-			   const XMLString& s) const {
+         const XMLString& s) const {
   return !((*this)==s);
 };
 
 // **********************************************************************
 XMLString& XMLString::operator+=(
-				 const XMLString& s) {
+         const XMLString& s) {
 
   unsigned long newLength = _length + s._length;
   char* newData = new char[newLength];
@@ -423,7 +423,7 @@ XMLString& XMLString::operator+=(
 
 // **********************************************************************
 long XMLString::loadFromFile(
-			     FILE* infile) {
+           FILE* infile) {
   if(_length) {
     delete _data;
   }
@@ -443,8 +443,8 @@ long XMLString::loadFromFile(
 
 // **********************************************************************
 void XMLString::insertString(
-			     const unsigned long& offset,
-			     const XMLString& s) {
+           const unsigned long& offset,
+           const XMLString& s) {
 
   if(offset > _length) {
     return;
@@ -478,8 +478,8 @@ void XMLString::insertString(
 
 // **********************************************************************
 void XMLString::deleteData(
-			   const unsigned long& offset,
-			   const unsigned long& count) {
+         const unsigned long& offset,
+         const unsigned long& count) {
 
   unsigned long i = offset + count;
   while(i < _length) {
@@ -501,9 +501,9 @@ void XMLString::deleteData(
 
 // **********************************************************************
 void XMLString::replaceData(
-			    const unsigned long& offset,
-			    unsigned long count,
-			    const XMLString& s) {
+          const unsigned long& offset,
+          unsigned long count,
+          const XMLString& s) {
 
   if(offset > _length) {
     *this += s;
@@ -543,9 +543,9 @@ void XMLString::replaceData(
 
 // **********************************************************************
 void XMLString::subString(
-			  XMLString& subS,
-			  const unsigned long& begin,
-			  const unsigned long& end_plus_one) const {
+        XMLString& subS,
+        const unsigned long& begin,
+        const unsigned long& end_plus_one) const {
 
   if(end_plus_one>_length) {
     throw XMLException(XMLException::RANGE_ERR);
@@ -596,8 +596,8 @@ bool XMLString::beginsWithXxMmLl() const {
     return 0;
   }
   return (((_data[0] == 'X') | (_data[0] == 'x'))
-	  &((_data[1] == 'M') | (_data[1] == 'm'))
-	  &((_data[2] == 'L') | (_data[2] == 'l')));
+    &((_data[1] == 'M') | (_data[1] == 'm'))
+    &((_data[2] == 'L') | (_data[2] == 'l')));
 };
 
 // **********************************************************************
@@ -606,8 +606,8 @@ bool XMLString::eqxml() const {
     return 0;
   }
   return (((_data[0] == 'X') | (_data[0] == 'x'))
-	  &((_data[1] == 'M') | (_data[1] == 'm'))
-	  &((_data[2] == 'L') | (_data[2] == 'l')));
+    &((_data[1] == 'M') | (_data[1] == 'm'))
+    &((_data[2] == 'L') | (_data[2] == 'l')));
 };
 
 // **********************************************************************
@@ -616,10 +616,10 @@ bool XMLString::eqxmlns() const {
     return 0;
   }
   return (((_data[0] == 'x') | (_data[0] == 'X'))
-	  &((_data[1] == 'm') | (_data[1] == 'M'))
-	  &((_data[2] == 'l') | (_data[2] == 'L'))
-	  &((_data[3] == 'n') | (_data[3] == 'N'))
-	  &((_data[4] == 's') | (_data[4] == 'S')));
+    &((_data[1] == 'm') | (_data[1] == 'M'))
+    &((_data[2] == 'l') | (_data[2] == 'L'))
+    &((_data[3] == 'n') | (_data[3] == 'N'))
+    &((_data[4] == 's') | (_data[4] == 'S')));
 };
 
 // **********************************************************************
@@ -648,11 +648,11 @@ bool XMLString::isNCName() const {
   }
   for(unsigned long i=1;i<_length;i++) {
     if(!XMLChar::isNCNameChar(_data[i])) {
-	return 0;
+  return 0;
     }
   }
   return 1;
-};	
+};  
 
 // **********************************************************************
 bool XMLString::isNSWellFormed() const {
@@ -669,8 +669,8 @@ bool XMLString::isNSWellFormed() const {
 bool XMLString::isVersionNum() const {
   for(unsigned long i=0;i<_length;i++) {
     if(!( XMLChar::isLatinLetter(_data[i])
-	  | XMLChar::isLatinDigit(_data[i])
-	  |(_data[i]=='_')|(_data[i]=='.')|(_data[i]==':')|(_data[i]=='-'))) {
+    | XMLChar::isLatinDigit(_data[i])
+    |(_data[i]=='_')|(_data[i]=='.')|(_data[i]==':')|(_data[i]=='-'))) {
       return 0;
     }
   }
@@ -687,8 +687,8 @@ bool XMLString::isEncName() const {
   }
   for(unsigned long i=1;i<_length;i++) {
     if(!( XMLChar::isLatinLetter(_data[i])
-	  | XMLChar::isLatinDigit(_data[i])
-	  |(_data[i]=='_')|(_data[i]=='.')|(_data[i]=='-'))) {
+    | XMLChar::isLatinDigit(_data[i])
+    |(_data[i]=='_')|(_data[i]=='.')|(_data[i]=='-'))) {
       return 0;
     }
   }
@@ -707,8 +707,8 @@ bool XMLString::isPubidLiteral() const {
 
 // **********************************************************************
 bool XMLString::splitNSName(
-			    XMLString& prefix,
-			    XMLString& localPart) const {
+          XMLString& prefix,
+          XMLString& localPart) const {
 
   unsigned long firstColon = 0;
 
@@ -738,9 +738,9 @@ bool XMLString::isAllWhiteSpace() const {
   return 1;
 };
 
-// ******************************************************************************
+// **************************************************************************
 bool XMLString::asULong(
-			unsigned long& outULong) const {
+      unsigned long& outULong) const {
 
   // checks to ensure string contains exactly one parsable
   // positive integer and parses it
@@ -783,9 +783,9 @@ bool XMLString::asULong(
   return 0;
 }
 
-// ******************************************************************************
+// **************************************************************************
 bool XMLString::asDouble(
-			 double& outDouble) const {
+       double& outDouble) const {
 
   // checks to ensure string contains exactly one parsable double and parses it
 
@@ -824,15 +824,15 @@ bool XMLString::asDouble(
     i++;
   }
 
-  if((_c_str[i]=='e')|(_c_str[i]=='e'))	{
+  if((_c_str[i]=='e')|(_c_str[i]=='e')) {
       // skip exponential
     i++;
     if((_c_str[i]=='+')|(_c_str[i]=='-')) {
-	// skip any +/- sign
+  // skip any +/- sign
       i++;
     }
     while(XMLChar::isLatinDigit(_c_str[i])) {
-	// skip any more numbers
+  // skip any more numbers
       i++;
     }
   }
@@ -876,5 +876,53 @@ void XMLString::goLatinAlphaNumeric() {
       _data[i]='_';
     }
   }
+}
+
+// **********************************************************************
+void XMLString::goLatinAlphaNumericNoUnderScore() {
+    // A function that converts the XML string to alpha numeric characters, and
+    // also removes any underscores, since in some cases even the underscore 
+    // character is unacceptable in the string.
+    
+  if(_length==0) {
+    return;
+  }
+
+  if(_c_str_valid) {
+    delete _c_str;
+  }
+  _c_str_valid=0;
+
+  // A first character digit or underscore isn't permissable, so remove it.
+  // Lacking a better way to do so, the removal is performed  by shifting each 
+  // subsequent character towards the front and shortening the array.  
+  if(XMLChar::isLatinDigit(_data[0])||_data[0] == '_') {
+    for(unsigned long i=0;i<_length;i++){
+        _data[i]=_data[i+1];
+    }
+    _length--;
+  }
+
+  // To get rid of the underscores by deleting them rather than replacing them
+  // when an underscore is encountered the contents of the character array are 
+  // shifted to the left from that point to the end, and the _length is reduced.  
+  for(unsigned long i=1; i<_length; i++) {
+    if(!(XMLChar::isLatinLetter(_data[i])|XMLChar::isLatinDigit(_data[i]))) {
+        for(unsigned long j=i;j<_length;j++){
+            _data[j]=_data[j+1];
+        }
+        _length--;
+        i--;
+    }
+  }
 };
 
+/*
+ * Local variables:
+ * c-indentation-style: bsd
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * End:
+ *
+ * vim: tabstop=2 expandtab shiftwidth=2:
+ */
